[
    ( ".scalafix.conf"             , "file_type_config" ),
    ( ".scalafmt.conf"             , "file_type_config" ),
    ( "build.properties"           , "file_type_config" ),
    ( "eslint.config.cjs"          , "file_type_eslint" ),
    ( "eslint.config.js"           , "file_type_eslint" ),
    ( "eslint.config.mjs"          , "file_type_eslint" ),
    ( "license"                    , "file_type_license"),
    ( "package-lock.json"          , "file_type_npm"    ),
    ( "package.json"               , "file_type_npm"    ),
    ( "readme"                     , "file_type_text"   ),
    ( "todo"                       , "file_type_todo"   ),
    ( "jenkinsfile"                , "file_type_jenkins"),
]
