[
( "emoji_type_lock", 0x1F512 ),
( "emoji_type_link", 0x1F517 ),
( "default_file", 0x100000 ),
( "default_folder_opened", 0x100001 ),
( "default_folder", 0x100002 ),
( "default_root_folder_opened", 0x100003 ),
( "default_root_folder", 0x100004 ),
( "file_type_access2", 0x100064 ),
( "file_type_access", 0x100065 ),
( "file_type_actionscript2", 0x100066 ),
( "file_type_actionscript", 0x100067 ),
( "file_type_ada", 0x100068 ),
( "file_type_advpl", 0x100069 ),
( "file_type_affectscript", 0x10006A ),
( "file_type_affinitydesigner", 0x10006B ),
( "file_type_affinityphoto", 0x10006C ),
( "file_type_affinitypublisher", 0x10006D ),
( "file_type_ai2", 0x10006E ),
( "file_type_ai", 0x10006F ),
( "file_type_al", 0x100070 ),
( "file_type_angular", 0x100071 ),
( "file_type_ansible", 0x100072 ),
( "file_type_antlr", 0x100073 ),
( "file_type_anyscript", 0x100074 ),
( "file_type_apache", 0x100075 ),
( "file_type_apex", 0x100076 ),
( "file_type_apib2", 0x100077 ),
( "file_type_apib", 0x100078 ),
( "file_type_api_extractor", 0x100079 ),
( "file_type_apl", 0x10007A ),
( "file_type_applescript", 0x10007B ),
( "file_type_appveyor", 0x10007C ),
( "file_type_arduino", 0x10007D ),
( "file_type_asciidoc", 0x10007E ),
( "file_type_asp", 0x10007F ),
( "file_type_aspx", 0x100080 ),
( "file_type_assembly", 0x100081 ),
( "file_type_ats", 0x100082 ),
( "file_type_audio", 0x100083 ),
( "file_type_aurelia", 0x100084 ),
( "file_type_autohotkey", 0x100085 ),
( "file_type_autoit", 0x100086 ),
( "file_type_avif", 0x100087 ),
( "file_type_avro", 0x100088 ),
( "file_type_awk", 0x100089 ),
( "file_type_aws", 0x10008A ),
( "file_type_azurepipelines", 0x10008B ),
( "file_type_azure", 0x10008C ),
( "file_type_babel2", 0x10008D ),
( "file_type_babel", 0x10008E ),
( "file_type_ballerina", 0x10008F ),
( "file_type_bats", 0x100090 ),
( "file_type_bat", 0x100091 ),
( "file_type_bazaar", 0x100092 ),
( "file_type_bazel", 0x100093 ),
( "file_type_befunge", 0x100094 ),
( "file_type_biml", 0x100095 ),
( "file_type_binary", 0x100096 ),
( "file_type_bitbucketpipeline", 0x100097 ),
( "file_type_bithound", 0x100098 ),
( "file_type_blade", 0x100099 ),
( "file_type_blitzbasic", 0x10009A ),
( "file_type_bolt", 0x10009B ),
( "file_type_bosque", 0x10009C ),
( "file_type_bower2", 0x10009D ),
( "file_type_bower", 0x10009E ),
( "file_type_browserslist", 0x10009F ),
( "file_type_buckbuild", 0x1000A0 ),
( "file_type_bundler", 0x1000A1 ),
( "file_type_c2", 0x1000A2 ),
( "file_type_c3", 0x1000A3 ),
( "file_type_cabal", 0x1000A4 ),
( "file_type_caddy", 0x1000A5 ),
( "file_type_cakephp", 0x1000A6 ),
( "file_type_cake", 0x1000A7 ),
( "file_type_c_al", 0x1000A8 ),
( "file_type_capacitor", 0x1000A9 ),
( "file_type_cargo", 0x1000AA ),
( "file_type_cddl", 0x1000AB ),
( "file_type_cert", 0x1000AC ),
( "file_type_ceylon", 0x1000AD ),
( "file_type_cf2", 0x1000AE ),
( "file_type_cfc2", 0x1000AF ),
( "file_type_cfc", 0x1000B0 ),
( "file_type_cfm2", 0x1000B1 ),
( "file_type_cfm", 0x1000B2 ),
( "file_type_cf", 0x1000B3 ),
( "file_type_cheader", 0x1000B4 ),
( "file_type_chef_cookbook", 0x1000B5 ),
( "file_type_chef", 0x1000B6 ),
( "file_type_circleci", 0x1000B7 ),
( "file_type_class", 0x1000B8 ),
( "file_type_clojurescript", 0x1000B9 ),
( "file_type_clojure", 0x1000BA ),
( "file_type_cloudfoundry", 0x1000BB ),
( "file_type_cmake", 0x1000BC ),
( "file_type_cobol", 0x1000BD ),
( "file_type_codacy", 0x1000BE ),
( "file_type_codeclimate", 0x1000BF ),
( "file_type_codecov", 0x1000C0 ),
( "file_type_codekit", 0x1000C1 ),
( "file_type_coffeelint", 0x1000C2 ),
( "file_type_coffeescript", 0x1000C3 ),
( "file_type_commitlint", 0x1000C4 ),
( "file_type_compass", 0x1000C5 ),
( "file_type_composer", 0x1000C6 ),
( "file_type_conan", 0x1000C7 ),
( "file_type_conda", 0x1000C8 ),
( "file_type_config", 0x1000C9 ),
( "file_type_confluence", 0x1000CA ),
( "file_type_coveralls", 0x1000CB ),
( "file_type_cpp2", 0x1000CC ),
( "file_type_cpp3", 0x1000CD ),
( "file_type_cppheader", 0x1000CE ),
( "file_type_cpp", 0x1000CF ),
( "file_type_crowdin", 0x1000D0 ),
( "file_type_crystal", 0x1000D1 ),
( "file_type_csharp2", 0x1000D2 ),
( "file_type_csharp", 0x1000D3 ),
( "file_type_csproj", 0x1000D4 ),
( "file_type_csscomb", 0x1000D5 ),
( "file_type_csslint", 0x1000D6 ),
( "file_type_cssmap", 0x1000D7 ),
( "file_type_css", 0x1000D8 ),
( "file_type_c", 0x1000D9 ),
( "file_type_cucumber", 0x1000DA ),
( "file_type_cuda", 0x1000DB ),
( "file_type_cvs", 0x1000DC ),
( "file_type_cypress", 0x1000DD ),
( "file_type_cython", 0x1000DE ),
( "file_type_dal", 0x1000DF ),
( "file_type_darcs", 0x1000E0 ),
( "file_type_dartlang", 0x1000E1 ),
( "file_type_db", 0x1000E2 ),
( "file_type_delphi", 0x1000E3 ),
( "file_type_dependabot", 0x1000E4 ),
( "file_type_dependencies", 0x1000E5 ),
( "file_type_devcontainer", 0x1000E6 ),
( "file_type_diff", 0x1000E7 ),
( "file_type_django", 0x1000E8 ),
( "file_type_dlang", 0x1000E9 ),
( "file_type_docker2", 0x1000EA ),
( "file_type_docker", 0x1000EB ),
( "file_type_dockertest2", 0x1000EC ),
( "file_type_dockertest", 0x1000ED ),
( "file_type_docpad", 0x1000EE ),
( "file_type_docz", 0x1000EF ),
( "file_type_dojo", 0x1000F0 ),
( "file_type_dotjs", 0x1000F1 ),
( "file_type_doxygen", 0x1000F2 ),
( "file_type_drawio", 0x1000F3 ),
( "file_type_drone", 0x1000F4 ),
( "file_type_drools", 0x1000F5 ),
( "file_type_dustjs", 0x1000F6 ),
( "file_type_dvc", 0x1000F7 ),
( "file_type_dylan", 0x1000F8 ),
( "file_type_edge2", 0x1000F9 ),
( "file_type_edge", 0x1000FA ),
( "file_type_editorconfig", 0x1000FB ),
( "file_type_eex", 0x1000FC ),
( "file_type_ejs", 0x1000FD ),
( "file_type_elasticbeanstalk", 0x1000FE ),
( "file_type_elastic", 0x1000FF ),
( "file_type_elixir", 0x100100 ),
( "file_type_elm2", 0x100101 ),
( "file_type_elm", 0x100102 ),
( "file_type_emacs", 0x100103 ),
( "file_type_ember", 0x100104 ),
( "file_type_ensime", 0x100105 ),
( "file_type_eps", 0x100106 ),
( "file_type_erb", 0x100107 ),
( "file_type_erlang2", 0x100108 ),
( "file_type_erlang", 0x100109 ),
( "file_type_eslint2", 0x10010A ),
( "file_type_eslint", 0x10010B ),
( "file_type_excel2", 0x10010C ),
( "file_type_excel", 0x10010D ),
( "file_type_expo", 0x10010E ),
( "file_type_falcon", 0x10010F ),
( "file_type_favicon", 0x100110 ),
( "file_type_fbx", 0x100111 ),
( "file_type_firebasehosting", 0x100112 ),
( "file_type_firebase", 0x100113 ),
( "file_type_firestore", 0x100114 ),
( "file_type_flash", 0x100115 ),
( "file_type_fla", 0x100116 ),
( "file_type_floobits", 0x100117 ),
( "file_type_flow", 0x100118 ),
( "file_type_flutter_package", 0x100119 ),
( "file_type_flutter", 0x10011A ),
( "file_type_font", 0x10011B ),
( "file_type_fortran", 0x10011C ),
( "file_type_fossa", 0x10011D ),
( "file_type_fossil", 0x10011E ),
( "file_type_freemarker", 0x10011F ),
( "file_type_fsharp2", 0x100120 ),
( "file_type_fsharp", 0x100121 ),
( "file_type_fsproj", 0x100122 ),
( "file_type_fthtml", 0x100123 ),
( "file_type_fusebox", 0x100124 ),
( "file_type_galen2", 0x100125 ),
( "file_type_galen", 0x100126 ),
( "file_type_gamemaker2", 0x100127 ),
( "file_type_gamemaker81", 0x100128 ),
( "file_type_gamemaker", 0x100129 ),
( "file_type_gatsby", 0x10012A ),
( "file_type_gcode", 0x10012B ),
( "file_type_genstat", 0x10012C ),
( "file_type_git2", 0x10012D ),
( "file_type_gitlab", 0x10012E ),
( "file_type_git", 0x10012F ),
( "file_type_glide", 0x100130 ),
( "file_type_glsl", 0x100131 ),
( "file_type_glyphs", 0x100132 ),
( "file_type_gnuplot", 0x100133 ),
( "file_type_go_aqua", 0x100134 ),
( "file_type_go_black", 0x100135 ),
( "file_type_godot", 0x100136 ),
( "file_type_go_fuchsia", 0x100137 ),
( "file_type_go_gopher", 0x100138 ),
( "file_type_go_lightblue", 0x100139 ),
( "file_type_go_package", 0x10013A ),
( "file_type_go", 0x10013B ),
( "file_type_go_white", 0x10013C ),
( "file_type_go_yellow", 0x10013D ),
( "file_type_gradle2", 0x10013E ),
( "file_type_gradle", 0x10013F ),
( "file_type_graphql_config", 0x100140 ),
( "file_type_graphql", 0x100141 ),
( "file_type_graphviz", 0x100142 ),
( "file_type_greenkeeper", 0x100143 ),
( "file_type_gridsome", 0x100144 ),
( "file_type_groovy2", 0x100145 ),
( "file_type_groovy", 0x100146 ),
( "file_type_grunt", 0x100147 ),
( "file_type_gulp", 0x100148 ),
( "file_type_haml", 0x100149 ),
( "file_type_handlebars2", 0x10014A ),
( "file_type_handlebars", 0x10014B ),
( "file_type_harbour", 0x10014C ),
( "file_type_haskell2", 0x10014D ),
( "file_type_haskell", 0x10014E ),
( "file_type_haxecheckstyle", 0x10014F ),
( "file_type_haxedevelop", 0x100150 ),
( "file_type_haxe", 0x100151 ),
( "file_type_helix", 0x100152 ),
( "file_type_helm", 0x100153 ),
( "file_type_hjson", 0x100154 ),
( "file_type_hlsl", 0x100155 ),
( "file_type_homeassistant", 0x100156 ),
( "file_type_host", 0x100157 ),
( "file_type_htmlhint", 0x100158 ),
( "file_type_html", 0x100159 ),
( "file_type_http", 0x10015A ),
( "file_type_hunspell", 0x10015B ),
( "file_type_husky", 0x10015C ),
( "file_type_hygen", 0x10015D ),
( "file_type_hy", 0x10015E ),
( "file_type_icl", 0x10015F ),
( "file_type_idrisbin", 0x100160 ),
( "file_type_idrispkg", 0x100161 ),
( "file_type_idris", 0x100162 ),
( "file_type_image", 0x100163 ),
( "file_type_imba", 0x100164 ),
( "file_type_inc", 0x100165 ),
( "file_type_infopath", 0x100166 ),
( "file_type_informix", 0x100167 ),
( "file_type_ini", 0x100168 ),
( "file_type_ink", 0x100169 ),
( "file_type_innosetup", 0x10016A ),
( "file_type_iodine", 0x10016B ),
( "file_type_ionic", 0x10016C ),
( "file_type_io", 0x10016D ),
( "file_type_jake", 0x10016E ),
( "file_type_janet", 0x10016F ),
( "file_type_jar", 0x100170 ),
( "file_type_jasmine", 0x100171 ),
( "file_type_java", 0x100172 ),
( "file_type_jbuilder", 0x100173 ),
( "file_type_jekyll", 0x100174 ),
( "file_type_jenkins", 0x100175 ),
( "file_type_jest_snapshot", 0x100176 ),
( "file_type_jest", 0x100177 ),
( "file_type_jinja", 0x100178 ),
( "file_type_jpm", 0x100179 ),
( "file_type_jsbeautify", 0x10017A ),
( "file_type_jsconfig", 0x10017B ),
( "file_type_jscpd", 0x10017C ),
( "file_type_jshint", 0x10017D ),
( "file_type_jsmap", 0x10017E ),
( "file_type_js_official", 0x10017F ),
( "file_type_json2", 0x100180 ),
( "file_type_json5", 0x100181 ),
( "file_type_jsonld", 0x100182 ),
( "file_type_jsonnet", 0x100183 ),
( "file_type_json_official", 0x100184 ),
( "file_type_json", 0x100185 ),
( "file_type_jsp", 0x100186 ),
( "file_type_jss", 0x100187 ),
( "file_type_js", 0x100188 ),
( "file_type_julia2", 0x100189 ),
( "#file_type_julia", 0x10018A ),
( "file_type_jupyter", 0x10018B ),
( "file_type_karma", 0x10018C ),
( "file_type_key", 0x10018D ),
( "file_type_kitchenci", 0x10018E ),
( "file_type_kite", 0x10018F ),
( "file_type_kivy", 0x100190 ),
( "file_type_kos", 0x100191 ),
( "file_type_kotlin", 0x100192 ),
( "file_type_kusto", 0x100193 ),
( "file_type_latino", 0x100194 ),
( "file_type_layout", 0x100195 ),
( "file_type_lerna", 0x100196 ),
( "file_type_less", 0x100197 ),
( "file_type_lex", 0x100198 ),
( "file_type_license", 0x100199 ),
( "file_type_light_actionscript2", 0x10019A ),
( "file_type_light_ada", 0x10019B ),
( "file_type_light_apl", 0x10019C ),
( "file_type_light_babel2", 0x10019D ),
( "file_type_light_babel", 0x10019E ),
( "file_type_light_cabal", 0x10019F ),
( "file_type_light_circleci", 0x1001A0 ),
( "file_type_light_cloudfoundry", 0x1001A1 ),
( "file_type_light_codacy", 0x1001A2 ),
( "file_type_light_codeclimate", 0x1001A3 ),
( "file_type_light_config", 0x1001A4 ),
( "file_type_light_crystal", 0x1001A5 ),
( "file_type_light_db", 0x1001A6 ),
( "file_type_light_docpad", 0x1001A7 ),
( "file_type_light_drone", 0x1001A8 ),
( "file_type_light_expo", 0x1001A9 ),
( "file_type_light_firebasehosting", 0x1001AA ),
( "file_type_light_fla", 0x1001AB ),
( "file_type_light_font", 0x1001AC ),
( "file_type_light_gamemaker2", 0x1001AD ),
( "file_type_light_gradle", 0x1001AE ),
( "file_type_light_hjson", 0x1001AF ),
( "file_type_lighthouse", 0x1001B0 ),
( "file_type_light_ini", 0x1001B1 ),
( "file_type_light_io", 0x1001B2 ),
( "file_type_light_jsconfig", 0x1001B3 ),
( "file_type_light_jsmap", 0x1001B4 ),
( "file_type_light_json5", 0x1001B5 ),
( "file_type_light_jsonld", 0x1001B6 ),
( "file_type_light_json", 0x1001B7 ),
( "file_type_light_js", 0x1001B8 ),
( "file_type_light_kite", 0x1001B9 ),
( "file_type_light_lerna", 0x1001BA ),
( "file_type_light_mdx", 0x1001BB ),
( "file_type_light_mlang", 0x1001BC ),
( "file_type_light_mustache", 0x1001BD ),
( "file_type_light_next", 0x1001BE ),
( "file_type_light_nim", 0x1001BF ),
( "file_type_light_openHAB", 0x1001C0 ),
( "file_type_light_pcl", 0x1001C1 ),
( "file_type_light_pnpm", 0x1001C2 ),
( "file_type_light_prettier", 0x1001C3 ),
( "file_type_light_prisma", 0x1001C4 ),
( "file_type_light_purescript", 0x1001C5 ),
( "file_type_light_razzle", 0x1001C6 ),
( "file_type_light_rehype", 0x1001C7 ),
( "file_type_light_remark", 0x1001C8 ),
( "file_type_light_retext", 0x1001C9 ),
( "file_type_light_rubocop", 0x1001CA ),
( "file_type_light_shaderlab", 0x1001CB ),
( "file_type_light_solidity", 0x1001CC ),
( "file_type_light_stylelint", 0x1001CD ),
( "file_type_light_stylus", 0x1001CE ),
( "file_type_light_symfony", 0x1001CF ),
( "file_type_light_systemd", 0x1001D0 ),
( "file_type_light_systemverilog", 0x1001D1 ),
( "file_type_light_testcafe", 0x1001D2 ),
( "file_type_light_testjs", 0x1001D3 ),
( "file_type_light_tex", 0x1001D4 ),
( "file_type_light_todo", 0x1001D5 ),
( "file_type_light_toml", 0x1001D6 ),
( "file_type_light_unibeautify", 0x1001D7 ),
( "file_type_light_vash", 0x1001D8 ),
( "file_type_light_vsixmanifest", 0x1001D9 ),
( "file_type_light_vsix", 0x1001DA ),
( "file_type_light_xfl", 0x1001DB ),
( "file_type_light_yaml", 0x1001DC ),
( "file_type_light_zeit", 0x1001DD ),
( "file_type_lime", 0x1001DE ),
( "file_type_lintstagedrc", 0x1001DF ),
( "file_type_liquid", 0x1001E0 ),
( "file_type_lisp", 0x1001E1 ),
( "file_type_livescript", 0x1001E2 ),
( "file_type_lnk", 0x1001E3 ),
( "file_type_locale", 0x1001E4 ),
( "file_type_log", 0x1001E5 ),
( "file_type_lolcode", 0x1001E6 ),
( "file_type_lsl", 0x1001E7 ),
( "file_type_lua", 0x1001E8 ),
( "file_type_lync", 0x1001E9 ),
( "file_type_makefile", 0x1001EA ),
( "file_type_manifest_bak", 0x1001EB ),
( "file_type_manifest_skip", 0x1001EC ),
( "file_type_manifest", 0x1001ED ),
( "file_type_map", 0x1001EE ),
( "file_type_mariadb", 0x1001EF ),
( "file_type_markdownlint", 0x1001F0 ),
( "file_type_markdown", 0x1001F1 ),
( "file_type_markojs", 0x1001F2 ),
( "file_type_marko", 0x1001F3 ),
( "file_type_matlab", 0x1001F4 ),
( "file_type_maven", 0x1001F5 ),
( "file_type_maxscript", 0x1001F6 ),
( "file_type_maya", 0x1001F7 ),
( "file_type_mdx", 0x1001F8 ),
( "file_type_mediawiki", 0x1001F9 ),
( "file_type_mercurial", 0x1001FA ),
( "file_type_meson", 0x1001FB ),
( "file_type_meteor", 0x1001FC ),
( "file_type_mjml", 0x1001FD ),
( "file_type_mlang", 0x1001FE ),
( "file_type_mocha", 0x1001FF ),
( "file_type_modernizr", 0x100200 ),
( "file_type_mojolicious", 0x100201 ),
( "file_type_moleculer", 0x100202 ),
( "file_type_mongo", 0x100203 ),
( "file_type_monotone", 0x100204 ),
( "file_type_mson", 0x100205 ),
( "file_type_mustache", 0x100206 ),
( "file_type_mysql", 0x100207 ),
( "file_type_nearly", 0x100208 ),
( "file_type_nest_adapter_js", 0x100209 ),
( "file_type_nest_adapter_ts", 0x10020A ),
( "file_type_nest_controller_js", 0x10020B ),
( "file_type_nest_controller_ts", 0x10020C ),
( "file_type_nest_decorator_js", 0x10020D ),
( "file_type_nest_decorator_ts", 0x10020E ),
( "file_type_nest_filter_js", 0x10020F ),
( "file_type_nest_filter_ts", 0x100210 ),
( "file_type_nest_gateway_js", 0x100211 ),
( "file_type_nest_gateway_ts", 0x100212 ),
( "file_type_nest_guard_js", 0x100213 ),
( "file_type_nest_guard_ts", 0x100214 ),
( "file_type_nest_interceptor_js", 0x100215 ),
( "file_type_nest_interceptor_ts", 0x100216 ),
( "file_type_nestjs", 0x100217 ),
( "file_type_nest_middleware_js", 0x100218 ),
( "file_type_nest_middleware_ts", 0x100219 ),
( "file_type_nest_module_js", 0x10021A ),
( "file_type_nest_module_ts", 0x10021B ),
( "file_type_nest_pipe_js", 0x10021C ),
( "file_type_nest_pipe_ts", 0x10021D ),
( "file_type_nest_service_js", 0x10021E ),
( "file_type_nest_service_ts", 0x10021F ),
( "file_type_netlify", 0x100220 ),
( "file_type_next", 0x100221 ),
( "file_type_ng_component_css", 0x100222 ),
( "file_type_ng_component_dart", 0x100223 ),
( "file_type_ng_component_html", 0x100224 ),
( "file_type_ng_component_js2", 0x100225 ),
( "file_type_ng_component_js", 0x100226 ),
( "file_type_ng_component_less", 0x100227 ),
( "file_type_ng_component_sass", 0x100228 ),
( "file_type_ng_component_scss", 0x100229 ),
( "file_type_ng_component_ts2", 0x10022A ),
( "file_type_ng_component_ts", 0x10022B ),
( "file_type_ng_controller_js", 0x10022C ),
( "file_type_ng_controller_ts", 0x10022D ),
( "file_type_ng_directive_dart", 0x10022E ),
( "file_type_ng_directive_js2", 0x10022F ),
( "file_type_ng_directive_js", 0x100230 ),
( "file_type_ng_directive_ts2", 0x100231 ),
( "file_type_ng_directive_ts", 0x100232 ),
( "file_type_ng_guard_dart", 0x100233 ),
( "file_type_ng_guard_js", 0x100234 ),
( "file_type_ng_guard_ts", 0x100235 ),
( "file_type_ng_interceptor_dart", 0x100236 ),
( "file_type_ng_interceptor_js", 0x100237 ),
( "file_type_ng_interceptor_ts", 0x100238 ),
( "file_type_nginx", 0x100239 ),
( "file_type_ng_module_dart", 0x10023A ),
( "file_type_ng_module_js2", 0x10023B ),
( "file_type_ng_module_js", 0x10023C ),
( "file_type_ng_module_ts2", 0x10023D ),
( "file_type_ng_module_ts", 0x10023E ),
( "file_type_ng_pipe_dart", 0x10023F ),
( "file_type_ng_pipe_js2", 0x100240 ),
( "file_type_ng_pipe_js", 0x100241 ),
( "file_type_ng_pipe_ts2", 0x100242 ),
( "file_type_ng_pipe_ts", 0x100243 ),
( "file_type_ng_routing_dart", 0x100244 ),
( "file_type_ng_routing_js2", 0x100245 ),
( "file_type_ng_routing_js", 0x100246 ),
( "file_type_ng_routing_ts2", 0x100247 ),
( "file_type_ng_routing_ts", 0x100248 ),
( "file_type_ng_service_dart", 0x100249 ),
( "file_type_ng_service_js2", 0x10024A ),
( "file_type_ng_service_js", 0x10024B ),
( "file_type_ng_service_ts2", 0x10024C ),
( "file_type_ng_service_ts", 0x10024D ),
( "file_type_ng_smart_component_dart", 0x10024E ),
( "file_type_ng_smart_component_js2", 0x10024F ),
( "file_type_ng_smart_component_js", 0x100250 ),
( "file_type_ng_smart_component_ts2", 0x100251 ),
( "file_type_ng_smart_component_ts", 0x100252 ),
( "file_type_ng_tailwind", 0x100253 ),
( "file_type_nimble", 0x100254 ),
( "file_type_nim", 0x100255 ),
( "file_type_ninja", 0x100256 ),
( "file_type_nix", 0x100257 ),
( "file_type_njsproj", 0x100258 ),
( "file_type_node2", 0x100259 ),
( "file_type_nodemon", 0x10025A ),
( "file_type_node", 0x10025B ),
( "file_type_npm", 0x10025C ),
( "file_type_nsi", 0x10025D ),
( "file_type_nsri-integrity", 0x10025E ),
( "file_type_nsri", 0x10025F ),
( "file_type_nuget", 0x100260 ),
( "file_type_numpy", 0x100261 ),
( "file_type_nunjucks", 0x100262 ),
( "file_type_nushell", 0xf07c6 ),
( "file_type_nuxt", 0x100263 ),
( "file_type_nyc", 0x100264 ),
( "file_type_objectivecpp", 0x100265 ),
( "file_type_objectivec", 0x100266 ),
( "file_type_ocaml", 0x100267 ),
( "file_type_onenote", 0x100268 ),
( "file_type_opencl", 0x100269 ),
( "file_type_openHAB", 0x10026A ),
( "file_type_org", 0x10026B ),
( "file_type_outlook", 0x10026C ),
( "file_type_ovpn", 0x10026D ),
( "file_type_package", 0x10026E ),
( "file_type_paket", 0x10026F ),
( "file_type_patch", 0x100270 ),
( "file_type_pcl", 0x100271 ),
( "file_type_pddl_happenings", 0x100272 ),
( "file_type_pddl_plan", 0x100273 ),
( "file_type_pddl", 0x100274 ),
( "file_type_pdf2", 0x100275 ),
( "file_type_pdf", 0x100276 ),
( "file_type_perl2", 0x100277 ),
( "file_type_perl6", 0x100278 ),
( "file_type_perl", 0x100279 ),
( "file_type_pgsql", 0x10027A ),
( "file_type_photoshop2", 0x10027B ),
( "file_type_photoshop", 0x10027C ),
( "file_type_php2", 0x10027D ),
( "file_type_php3", 0x10027E ),
( "file_type_phpcsfixer", 0x10027F ),
( "file_type_php", 0x100280 ),
( "file_type_phpunit", 0x100281 ),
( "file_type_phraseapp", 0x100282 ),
( "file_type_pine", 0x100283 ),
( "file_type_pip", 0x100284 ),
( "file_type_plantuml", 0x100285 ),
( "file_type_platformio", 0x100286 ),
( "file_type_plsql_package_body", 0x100287 ),
( "file_type_plsql_package_header", 0x100288 ),
( "file_type_plsql_package_spec", 0x100289 ),
( "file_type_plsql_package", 0x10028A ),
( "file_type_plsql", 0x10028B ),
( "file_type_pnpm", 0x10028C ),
( "file_type_poedit", 0x10028D ),
( "file_type_polymer", 0x10028E ),
( "file_type_pony", 0x10028F ),
( "file_type_postcssconfig", 0x100290 ),
( "file_type_postcss", 0x100291 ),
( "file_type_powerpoint2", 0x100292 ),
( "file_type_powerpoint", 0x100293 ),
( "file_type_powershell2", 0x100294 ),
( "file_type_powershell_format", 0x100295 ),
( "file_type_powershell_psd2", 0x100296 ),
( "file_type_powershell_psd", 0x100297 ),
( "file_type_powershell_psm2", 0x100298 ),
( "file_type_powershell_psm", 0x100299 ),
( "file_type_powershell", 0x10029A ),
( "file_type_powershell_types", 0x10029B ),
( "file_type_precommit", 0x10029C ),
( "file_type_prettier", 0x10029D ),
( "file_type_prisma", 0x10029E ),
( "file_type_processinglang", 0x10029F ),
( "file_type_procfile", 0x1002A0 ),
( "file_type_progress", 0x1002A1 ),
( "file_type_prolog", 0x1002A2 ),
( "file_type_prometheus", 0x1002A3 ),
( "file_type_protobuf", 0x1002A4 ),
( "file_type_protractor", 0x1002A5 ),
( "file_type_publisher", 0x1002A6 ),
( "file_type_pug", 0x1002A7 ),
( "file_type_puppet", 0x1002A8 ),
( "file_type_purescript", 0x1002A9 ),
( "file_type_pyret", 0x1002AA ),
( "file_type_python", 0x1002AB ),
( "file_type_pyup", 0x1002AC ),
( "file_type_qbs", 0x1002AD ),
( "file_type_qlikview", 0x1002AE ),
( "file_type_qmldir", 0x1002AF ),
( "file_type_qml", 0x1002B0 ),
( "file_type_qsharp", 0x1002B1 ),
( "file_type_q", 0x1002B2 ),
( "file_type_quasar", 0x1002B3 ),
( "file_type_racket", 0x1002B4 ),
( "file_type_rails", 0x1002B5 ),
( "file_type_rake", 0x1002B6 ),
( "file_type_raml", 0x1002B7 ),
( "file_type_razor", 0x1002B8 ),
( "file_type_razzle", 0x1002B9 ),
( "file_type_reactjs", 0x1002BA ),
( "file_type_reacttemplate", 0x1002BB ),
( "file_type_reactts", 0x1002BC ),
( "file_type_reason", 0x1002BD ),
( "file_type_red", 0x1002BE ),
( "file_type_registry", 0x1002BF ),
( "file_type_rego", 0x1002C0 ),
( "file_type_rehype", 0x1002C1 ),
( "file_type_remark", 0x1002C2 ),
( "file_type_renovate", 0x1002C3 ),
( "file_type_rescript", 0x1002C4 ),
( "file_type_rest", 0x1002C5 ),
( "file_type_retext", 0x1002C6 ),
( "file_type_rexx", 0x1002C7 ),
( "file_type_riot", 0x1002C8 ),
( "file_type_rmd", 0x1002C9 ),
( "file_type_robotframework", 0x1002CA ),
( "file_type_robots", 0x1002CB ),
( "file_type_rollup", 0x1002CC ),
( "file_type_rproj", 0x1002CD ),
( "file_type_rspec", 0x1002CE ),
( "file_type_r", 0x1002CF ),
( "file_type_rubocop", 0x1002D0 ),
( "file_type_ruby", 0x1002D1 ),
( "file_type_rust", 0x1002D2 ),
( "file_type_rust_toolchain", 0x1002D3 ),
( "file_type_sails", 0x1002D4 ),
( "file_type_saltstack", 0x1002D5 ),
( "file_type_san", 0x1002D6 ),
( "file_type_sass", 0x1002D7 ),
( "file_type_sas", 0x1002D8 ),
( "file_type_sbt", 0x1002D9 ),
( "file_type_scala", 0x1002DA ),
( "file_type_scilab", 0x1002DB ),
( "file_type_script", 0x1002DC ),
( "file_type_scss2", 0x1002DD ),
( "file_type_scss", 0x1002DE ),
( "file_type_sdlang", 0x1002DF ),
( "file_type_sentry", 0x1002E0 ),
( "file_type_sequelize", 0x1002E1 ),
( "file_type_serverless", 0x1002E2 ),
( "file_type_shaderlab", 0x1002E3 ),
( "file_type_shell", 0x1002E4 ),
( "file_type_silverstripe", 0x1002E5 ),
( "file_type_sketch", 0x1002E6 ),
( "file_type_skipper", 0x1002E7 ),
( "file_type_slang", 0x1002E8 ),
( "file_type_slice", 0x1002E9 ),
( "file_type_slim", 0x1002EA ),
( "file_type_sln2", 0x1002EB ),
( "file_type_sln", 0x1002EC ),
( "file_type_smarty", 0x1002ED ),
( "file_type_snapcraft", 0x1002EE ),
( "file_type_snort", 0x1002EF ),
( "file_type_snyk", 0x1002F0 ),
( "file_type_solidarity", 0x1002F1 ),
( "file_type_solidity", 0x1002F2 ),
( "file_type_source", 0x1002F3 ),
( "file_type_spacengine", 0x1002F4 ),
( "file_type_sqf", 0x1002F5 ),
( "file_type_sqlite", 0x1002F6 ),
( "file_type_sql", 0x1002F7 ),
( "file_type_squirrel", 0x1002F8 ),
( "file_type_sss", 0x1002F9 ),
( "file_type_stan", 0x1002FA ),
( "file_type_stata", 0x1002FB ),
( "file_type_stencil", 0x1002FC ),
( "file_type_storyboard", 0x1002FD ),
( "file_type_storybook", 0x1002FE ),
( "file_type_stylable", 0x1002FF ),
( "file_type_styled", 0x100300 ),
( "file_type_stylelint", 0x100301 ),
( "file_type_style", 0x100302 ),
( "file_type_stylish_haskell", 0x100303 ),
( "file_type_stylus", 0x100304 ),
( "file_type_subversion", 0x100305 ),
( "file_type_svelte", 0x100306 ),
( "file_type_svg", 0x100307 ),
( "file_type_swagger", 0x100308 ),
( "file_type_swift", 0x100309 ),
( "file_type_swig", 0x10030A ),
( "file_type_symfony", 0x10030B ),
( "file_type_systemd", 0x10030C ),
( "file_type_systemverilog", 0x10030D ),
( "file_type_t4tt", 0x10030E ),
( "file_type_tailwind", 0x10030F ),
( "file_type_tcl", 0x100310 ),
( "file_type_tera", 0x100311 ),
( "file_type_terraform", 0x100312 ),
( "file_type_testcafe", 0x100313 ),
( "file_type_testjs", 0x100314 ),
( "file_type_test", 0x100315 ),
( "file_type_testts", 0x100316 ),
( "file_type_tex", 0x100317 ),
( "file_type_textile", 0x100318 ),
( "file_type_text", 0x100319 ),
( "file_type_tfs", 0x10031A ),
( "file_type_todo", 0x10031B ),
( "file_type_toml", 0x10031C ),
( "file_type_tox", 0x10031D ),
( "file_type_travis", 0x10031E ),
( "file_type_tsconfig", 0x10031F ),
( "file_type_tslint", 0x100320 ),
( "file_type_ttcn", 0x100321 ),
( "file_type_tt", 0x100322 ),
( "file_type_twig", 0x100323 ),
( "file_type_typescriptdef_official", 0x100324 ),
( "file_type_typescriptdef", 0x100325 ),
( "file_type_typescript_official", 0x100326 ),
( "file_type_typescript", 0x100327 ),
( "file_type_typo3", 0x100328 ),
( "file_type_unibeautify", 0x100329 ),
( "file_type_vagrant", 0x10032A ),
( "file_type_vala", 0x10032B ),
( "file_type_vapi", 0x10032C ),
( "file_type_vash", 0x10032D ),
( "file_type_vba", 0x10032E ),
( "file_type_vbhtml", 0x10032F ),
( "file_type_vbproj", 0x100330 ),
( "file_type_vb", 0x100331 ),
( "file_type_vcxproj", 0x100332 ),
( "file_type_velocity", 0x100333 ),
( "file_type_verilog", 0x100334 ),
( "file_type_vhdl", 0x100335 ),
( "file_type_video", 0x100336 ),
( "file_type_view", 0x100337 ),
( "file_type_vim", 0x100338 ),
( "file_type_vlang", 0x100339 ),
( "file_type_volt", 0x10033A ),
( "file_type_vscode2", 0x10033B ),
( "file_type_vscode3", 0x10033C ),
( "file_type_vscode-insiders", 0x10033D ),
( "file_type_vscode", 0x10033E ),
( "file_type_vsixmanifest", 0x10033F ),
( "file_type_vsix", 0x100340 ),
( "file_type_vueconfig", 0x100341 ),
( "file_type_vue", 0x100342 ),
( "file_type_wallaby", 0x100343 ),
( "file_type_wasm", 0x100344 ),
( "file_type_watchmanconfig", 0x100345 ),
( "file_type_webpack", 0x100346 ),
( "file_type_webp", 0x100347 ),
( "file_type_wenyan", 0x100348 ),
( "file_type_wercker", 0x100349 ),
( "file_type_wolfram", 0x10034A ),
( "file_type_word2", 0x10034B ),
( "file_type_word", 0x10034C ),
( "file_type_wpml", 0x10034D ),
( "file_type_wurst", 0x10034E ),
( "file_type_wxml", 0x10034F ),
( "file_type_wxss", 0x100350 ),
( "file_type_xcode", 0x100351 ),
( "file_type_xfl", 0x100352 ),
( "file_type_xib", 0x100353 ),
( "file_type_xliff", 0x100354 ),
( "file_type_xmake", 0x100355 ),
( "file_type_xml", 0x100356 ),
( "file_type_xquery", 0x100357 ),
( "file_type_xsl", 0x100358 ),
( "file_type_yacc", 0x100359 ),
( "file_type_yamllint", 0x10035A ),
( "file_type_yaml", 0x10035B ),
( "file_type_yandex", 0x10035C ),
( "file_type_yang", 0x10035D ),
( "file_type_yarn", 0x10035E ),
( "file_type_yeoman", 0x10035F ),
( "file_type_zeit", 0x100360 ),
( "file_type_zig", 0x100361 ),
( "file_type_zip2", 0x100362 ),
( "file_type_zip", 0x100363 ),
]
