// if you want to find an icon for a file: go to https://www.nerdfonts.com/cheat-sheet and search
// for f07b in case of "default_file". That should return: 
[
( "default_file", 0xf15b ), // 
( "default_folder", 0xf07b ), // 
( "default_folder_opened", 0xf07c ), // 
( "default_root_folder", 0xea83 ), // 
( "default_root_folder_opened", 0xf115 ), // 
( "emoji_type_link", 0xf0c1 ), // 
( "emoji_type_lock", 0xf023 ), // 
( "file_type_access", 0xf0221 ), // 󰈡
( "file_type_access2", 0xf1aa1 ), // 󱪡
( "file_type_actionscript", 0xeaff ), // 
( "file_type_actionscript2", 0xeaff ), // 
( "file_type_ada", 0xf15b ),
( "file_type_advpl", 0xf15b ),
( "file_type_affectscript", 0xf0477 ), // 󰑷
( "file_type_affinitydesigner", 0xf1c5 ), // 
( "file_type_affinityphoto", 0xf1c5 ), // 
( "file_type_affinitypublisher", 0xf1c5 ), // 
( "file_type_ai", 0xe669 ), // 
( "file_type_ai2", 0xe7b4 ), // 
( "file_type_al", 0xf061d ), // 󰘝
( "file_type_angular", 0xe753 ), // 
( "file_type_ansible", 0xf109a ), // 󱂚
( "file_type_antlr", 0xf1119 ), // 󱄙
( "file_type_anyscript", 0xf0477 ), // 󰑷
( "file_type_apache", 0xf048b ), // 󰒋
( "file_type_apex", 0xf17b ), // 
( "file_type_api_extractor", 0xf109b ), // 󱂛
( "file_type_apib", 0xf109b ), // 󱂛
( "file_type_apib2", 0xf109b ), // 󱂛
( "file_type_apl", 0xe638 ), // 
( "file_type_applescript", 0xe711 ), // 
( "file_type_appveyor", 0xeacf ), // 
( "file_type_arduino", 0xf043f ), // 󰐿
( "file_type_asciidoc", 0xf09ee ), // 󰧮
( "file_type_asp", 0xf0aae ), // 󰪮
( "file_type_aspx", 0xf0aae ), // 󰪮
( "file_type_assembly", 0xe637 ), // 
( "file_type_ats", 0xf107c ), // 󱁼
( "file_type_audio", 0xf0223 ), // 󰈣
( "file_type_aurelia", 0xf022a ), // 󰈪
( "file_type_autohotkey", 0xf107b ), // 󰷊
( "file_type_autoit", 0xf107b ), // 󰷊
( "file_type_avif", 0xf021f ), // 󰈟
( "file_type_avro", 0xf09ee ), // 󰧮
( "file_type_awk", 0xf09ee ), // 󰧮
( "file_type_aws", 0xf102a ), // 󱀪
( "file_type_azure", 0xf102a ), // 󱀪
( "file_type_azurepipelines", 0xf102a ), // 󱀪
( "file_type_babel", 0xf0a25 ), // 󰨥
( "file_type_babel2", 0xf0a25 ), // 󰨥
( "file_type_ballerina", 0xf15ca ), // 󱗊
( "file_type_bat", 0xf0b5f ), // 󰭟
( "file_type_bats", 0xf0b5f ), // 󰭟
( "file_type_bazaar", 0xf0d6 ), // 
( "file_type_bazel", 0xe63a ), // 
( "file_type_befunge", 0xf04cc ), // 󰓌
( "file_type_biml", 0xe73e ), // 
( "file_type_binary", 0xeae8 ), // 
( "file_type_bitbucketpipeline", 0xe703 ), // 
( "file_type_bithound", 0xf0a44 ), // 󰩄
( "file_type_blade", 0xf0e61 ), // 󰹡
( "file_type_blitzbasic", 0xf0e7 ), // 
( "file_type_bolt", 0xf0e7 ), // 
( "file_type_bosque", 0xf021b ), // 󰈛
( "file_type_bower", 0xe74d ), // 
( "file_type_bower2", 0xe74d ), // 
( "file_type_browserslist", 0xf488 ), // 
( "file_type_buckbuild", 0xf1415 ), // 󱐕
( "file_type_bundler", 0xf107c ), // 󱁼
( "file_type_c", 0xe649 ), // 
( "file_type_c2", 0xf107c ), // 󱁼
( "file_type_c3", 0xf107c ), // 󱁼
( "file_type_c_al", 0xf107c ), // 󱁼
( "file_type_cabal", 0xe777 ), // 
( "file_type_caddy", 0xf107c ), // 󱁼
( "file_type_cake", 0xe63d ), // 
( "file_type_cakephp", 0xe63d ), // 
( "file_type_capacitor", 0xf107c ), // 󱁼
( "file_type_cargo", 0xe68b ), // 
( "file_type_cddl", 0xf107c ), // 󱁼
( "file_type_cert", 0xf1186 ), // 󱆆
( "file_type_ceylon", 0xf107c ), // 󱁼
( "file_type_cf", 0xf107c ), // 󱁼
( "file_type_cf2", 0xf107c ), // 󱁼
( "file_type_cfc", 0xf107c ), // 󱁼
( "file_type_cfc2", 0xf107c ), // 󱁼
( "file_type_cfm", 0xf107c ), // 󱁼
( "file_type_cfm2", 0xf107c ), // 󱁼
( "file_type_cheader", 0xf0273 ), // 󰉳
( "file_type_chef", 0xf107c ), // 󱁼
( "file_type_chef_cookbook", 0xf107c), // 󱁼
( "file_type_circleci", 0xf107c ), // 󱁼
( "file_type_class", 0xeb5b ), // 
( "file_type_clojure", 0xe642 ), // 
( "file_type_clojurescript", 0xe768 ), // 
( "file_type_cloudfoundry", 0xf0217 ), // 󰈗
( "file_type_cmake", 0xe673 ), // 
( "file_type_cobol", 0xf107c ), // 󱁼
( "file_type_codacy", 0xf017c ), // 󱁼
( "file_type_codeclimate", 0xf017c ), // 󱁼
( "file_type_codecov", 0xf017c ), // 󱁼
( "file_type_codekit", 0xeae9 ), // 
( "file_type_coffeelint", 0xf06ca ), // 󰛊
( "file_type_coffeescript", 0xe751 ), // 
( "file_type_commitlint", 0xe729 ), // 
( "file_type_compass", 0xebd5 ), // 
( "file_type_composer", 0xe783 ), // 
( "file_type_conan", 0xf107c ), // 󱁼
( "file_type_conda", 0xf107c ), // 󱁼
( "file_type_config", 0xe615 ), // 
( "file_type_confluence", 0xf0303 ), // 󰌃
( "file_type_coveralls", 0xf107c ), // 󱁼
( "file_type_cpp", 0xe646 ), // 
( "file_type_cpp2", 0xe61d ), // 
( "file_type_cpp3", 0xf0672 ), // 󰙲
( "file_type_cppheader", 0xf0273 ), // 󰉳
( "file_type_crowdin", 0xf1975 ), // 󱥵
( "file_type_crystal", 0xe62f ), // 
( "file_type_csharp", 0xe648 ), // 
( "file_type_csharp2", 0xe648 ), // 
( "file_type_csproj", 0xeb51 ), // 
( "file_type_css", 0xe749), // 
( "file_type_csscomb", 0xe614 ), // 
( "file_type_csslint", 0xe614 ), // 
( "file_type_cssmap", 0xe614 ), // 
( "file_type_csv", 0xe64a ), // 
( "file_type_cucumber", 0xf017c ), // 󱁼
( "file_type_cuda", 0xe64b ), // 
( "file_type_cvs", 0xf017c ), // 󱁼
( "file_type_cypress", 0xf0c35 ), // 󰰵
( "file_type_cython", 0xe73c ), // 
( "file_type_dal", 0xf15b ),
( "file_type_darcs", 0xf15b ),
( "file_type_dartlang", 0xe798 ), // 
( "file_type_db", 0xe64d ), // 
( "file_type_delphi", 0xf15b ),
( "file_type_dependabot", 0xf0573 ), // 󰕳
( "file_type_dependencies", 0xf0573 ), // 󰕳
( "file_type_devcontainer", 0xf15b ),
( "file_type_diff", 0xf055a ), // 󰕚
( "file_type_django", 0xe71d ), // 
( "file_type_dlang", 0xe7af ), // 
( "file_type_docker", 0xf308 ), // 
( "file_type_docker2", 0xe7b0 ), // 
( "file_type_dockertest", 0xe650 ), // 
( "file_type_dockertest2", 0xf0868 ), // 󰡨
( "file_type_docpad", 0xf1a9a ), // 󱪚
( "file_type_docz", 0xf0dc9 ), // 󰷉
( "file_type_dojo", 0xe71c ), // 
( "file_type_dotjs", 0xf444 ), // 
( "file_type_doxygen", 0xf15b ),
( "file_type_drawio", 0xf0f49 ), // 󰽉
( "file_type_drone", 0xf01e2 ), // 󰇢
( "file_type_drools", 0xf15b ),
( "file_type_dustjs", 0xe35d ), // 
( "file_type_dvc", 0xf15b ),
( "file_type_dylan", 0xf15b ),
( "file_type_edge", 0xf01e9 ), // 󰇩
( "file_type_edge2", 0xf282 ), // 
( "file_type_editorconfig", 0xe652 ), // 
( "file_type_eex", 0xf15b ),
( "file_type_ejs", 0xe618 ), // 
( "file_type_elastic", 0xea6d ), // 
( "file_type_elasticbeanstalk", 0xe26a ), // 
( "file_type_elixir", 0xe62d ), // 
( "file_type_elm", 0xe62c ), // 
( "file_type_elm2", 0xe62c ), // 
( "file_type_emacs", 0xe632 ), // 
( "file_type_ember", 0xe71b ), // 
( "file_type_ensime", 0xf15b ),
( "file_type_env", 0xe615 ), // 
( "file_type_eps", 0xf10e0 ), // 󱃠
( "file_type_erb", 0xf15b ),
( "file_type_erlang", 0xe7b1 ), // 
( "file_type_erlang2", 0xe7b1 ), // 
( "file_type_eslint", 0xf0c7a ), // 󰱺
( "file_type_eslint2", 0xf0c7a ), // 󰱺
( "file_type_excel", 0xf021b ), // 󰈛
( "file_type_excel2", 0xf021b ), // 󰈛
( "file_type_expo", 0xe27c ), // 
( "file_type_falcon", 0xf15b ),
( "file_type_favicon", 0xe623 ), // 
( "file_type_fbx", 0xf15b ),
( "file_type_fennel", 0xe6af ), // 
( "file_type_firebase", 0xf0967 ), // 󰥧
( "file_type_firebasehosting", 0xe787 ), // 
( "file_type_firestore", 0xe657 ), // 
( "file_type_fla", 0xf15b ),
( "file_type_flash", 0xf0e7 ), // 
( "file_type_floobits", 0xf15b ),
( "file_type_flow", 0xf15b ),
( "file_type_flutter", 0xf15b ),
( "file_type_flutter_package", 0xf487 ), // 
( "file_type_font", 0xf031 ), // 
( "file_type_fortran", 0xf121a ), // 󱈚
( "file_type_fossa", 0xf15b ),
( "file_type_fossil", 0xf15b ),
( "file_type_freemarker", 0xf15b ),
( "file_type_fsharp", 0xe65a ), // 
( "file_type_fsharp2", 0xe65a ), // 
( "file_type_fsproj", 0xe65a ), // 
( "file_type_fthtml", 0xf0f6d ), // 󰽭
( "file_type_fusebox", 0xf15b ),
( "file_type_galen", 0xf15b ),
( "file_type_galen2", 0xf15b ),
( "file_type_gamemaker", 0xf0eb7 ), // 󰺷
( "file_type_gamemaker2", 0xf1393 ), // 󱎓
( "file_type_gamemaker81", 0xf11b ), // 
( "file_type_gatsby", 0xf0e43 ), // 󰹃
( "file_type_gcode", 0xf15b ),
( "file_type_genstat", 0xf15b ),
( "file_type_git", 0xe702 ), // 
( "file_type_git2", 0xe702 ), // 
( "file_type_gitlab", 0xf296 ), // 
( "file_type_glide", 0xf2a5 ), // 
( "file_type_glsl", 0xf0dcb ), // 󰷋
( "file_type_glyphs", 0xf15b ),
( "file_type_gnuplot", 0xe779 ), // 
( "file_type_go", 0xe626 ), // 
( "file_type_go_aqua", 0xf15b ),
( "file_type_go_black", 0xf15b ),
( "file_type_go_fuchsia", 0xe626 ), // 
( "file_type_go_gopher", 0xe626 ), // 
( "file_type_go_lightblue", 0xe626 ), // 
( "file_type_go_package", 0xe626 ), // 
( "file_type_go_white", 0xe626 ), // 
( "file_type_go_yellow", 0xe626 ), // 
( "file_type_godot", 0xe65f ), // 
( "file_type_gradle", 0xe660 ), // 
( "file_type_gradle2", 0xe660 ), // 
( "file_type_graphql", 0xf0877 ), // 󰡷
( "file_type_graphql_config", 0xf0877 ), // 󰡷
( "file_type_graphviz", 0xeb03 ), // 
( "file_type_greenkeeper", 0xf15b ),
( "file_type_gridsome", 0xf15b ),
( "file_type_groovy", 0xe775 ), // 
( "file_type_groovy2", 0xe775 ), // 
( "file_type_grunt", 0xe74c ), // 
( "file_type_gulp", 0xe763 ), // 
( "file_type_haml", 0xe664 ), // 
( "file_type_handlebars", 0xe60e ), // 
( "file_type_handlebars2", 0xe60e ), // 
( "file_type_harbour", 0xf15b ),
( "file_type_haskell", 0xe777 ), // 
( "file_type_haskell2", 0xe777 ), // 
( "file_type_haxe", 0xe666 ), // 
( "file_type_haxecheckstyle", 0xe666 ), // 
( "file_type_haxedevelop", 0xe666 ), // 
( "file_type_helix", 0xf15b ),
( "file_type_helm", 0xf15b ),
( "file_type_hjson", 0xe60b ), // 
( "file_type_hlsl", 0xf0dcb ), // 󰷋
( "file_type_homeassistant", 0xf07d0 ), // 󰟐
( "file_type_host", 0xe0a2 ), // 
( "file_type_htaccess", 0xe615 ), // 
( "file_type_html", 0xe736  ), // 
( "file_type_htmlhint", 0xe736 ), // 
( "file_type_http", 0xf15b ),
( "file_type_hunspell", 0xf15b ),
( "file_type_husky", 0xf107c ),
( "file_type_hy", 0xf15b ),
( "file_type_hygen", 0xf15b ),
( "file_type_icl", 0xf15b ),
( "file_type_idris", 0xf15b ),
( "file_type_idrisbin", 0xf15b ),
( "file_type_idrispkg", 0xf15b ),
( "file_type_image", 0xf021f ), // 󰈟
( "file_type_imba", 0xf15b ),
( "file_type_inc", 0xf15b ),
( "file_type_infopath", 0xf15b ),
( "file_type_informix", 0xf15b ),
( "file_type_ini", 0xf15b ),
( "file_type_ink", 0xf15b ),
( "file_type_innosetup", 0xf15b ),
( "file_type_io", 0xf15b ),
( "file_type_iodine", 0xf15b ),
( "file_type_ionic", 0xf15b ),
( "file_type_jake", 0xf15b ),
( "file_type_janet", 0xf15b ),
( "file_type_jar", 0xe256 ), // 
( "file_type_jasmine", 0xf15b ),
( "file_type_java", 0xe256 ), // 
( "file_type_jbuilder", 0xe256 ), // 
( "file_type_jekyll", 0xe70d ), // 
( "file_type_jenkins", 0xe767 ), // 
( "file_type_jest", 0xf0668 ), // 󰙨
( "file_type_jest_snapshot", 0xf0668 ), // 󰙨
( "file_type_jinja", 0xe66f ), // 
( "file_type_jpm", 0xf15b ),
( "file_type_js", 0xf031e ), // 
( "file_type_js_official", 0xf031e ), // 
( "file_type_jsbeautify", 0xf031e ), // 
( "file_type_jsconfig", 0xf031e ), // 
( "file_type_jscpd", 0xf031e ), // 
( "file_type_jshint", 0xf031e ), // 
( "file_type_jsmap", 0xf031e ), // 
( "file_type_json", 0xe60b ), // 
( "file_type_json2", 0xe60b ), // 
( "file_type_json5", 0xe60b ), // 
( "file_type_json_official", 0xe60b ), // 
( "file_type_jsonld", 0xf0626 ), // 󰘦
( "file_type_jsonnet", 0xf0626 ), // 󰘦
( "file_type_jsp", 0xf15b ),
( "file_type_jss", 0xf15b ),
( "file_type_julia", 0xe624 ), // 
( "file_type_julia2", 0xe624 ),  // 
( "file_type_jupyter", 0x10018B ),
( "file_type_karma", 0xf15b ),
( "file_type_key", 0xf1184 ), // 󱆄
( "file_type_kitchenci", 0xf15b ),
( "file_type_kite", 0xf15b ),
( "file_type_kivy", 0xf15b ),
( "file_type_kos", 0xf15b ),
( "file_type_kotlin", 0xe634 ), // 
( "file_type_kusto", 0xf15b ),
( "file_type_latino", 0xf15b ),
( "file_type_layout", 0xf15b ),
( "file_type_lerna", 0xf15b ),
( "file_type_less", 0xe758 ), // 
( "file_type_lex", 0xf15b ),
( "file_type_license", 0xe60a ), // 
( "file_type_light_actionscript2", 0xf0477 ), // 󰑷
( "file_type_light_ada", 0xf15b ),
( "file_type_light_apl", 0xf15b ),
( "file_type_light_babel", 0xf0a25 ), // 󰨥
( "file_type_light_babel2", 0xf0a25 ), // 󰨥
( "file_type_light_cabal", 0xf15b ),
( "file_type_light_circleci", 0xf15b ),
( "file_type_light_cloudfoundry", 0xf15b ),
( "file_type_light_codacy", 0xf15b ),
( "file_type_light_codeclimate", 0xf15b ),
( "file_type_light_config", 0xf107c ), // 󱁼
( "file_type_light_crystal", 0xe62f ), // 
( "file_type_light_db", 0xe64d ), // 
( "file_type_light_docpad", 0xf1a9a ), // 󱪚
( "file_type_light_drone", 0xf01e2 ), // 󰇢
( "file_type_light_expo", 0xe27c ), // 
( "file_type_light_firebasehosting", 0xe787 ), // 
( "file_type_light_fla", 0xf15b ),
( "file_type_light_font", 0xf031 ), // 
( "file_type_light_gamemaker2", 0xf1393 ), // 󱎓
( "file_type_light_gradle", 0xe738 ), // 
( "file_type_light_hjson", 0xe60b ), // 
( "file_type_light_ini", 0xf15b ),
( "file_type_light_io", 0xf15b ),
( "file_type_light_js", 0xf031e ), // 
( "file_type_light_jsconfig", 0xf031e ), // 
( "file_type_light_jsmap", 0xf031e ), // 
( "file_type_light_json", 0xe60b ), // 
( "file_type_light_json5", 0xe60b ), // 
( "file_type_light_jsonld", 0xf0626 ), // 󰘦
( "file_type_light_kite", 0xf15b ),
( "file_type_light_lerna", 0xf15b ),
( "file_type_light_mdx", 0xe73e ), // 
( "file_type_light_mlang", 0xf15b ),
( "file_type_light_mustache", 0xe228 ), // 
( "file_type_light_next", 0xf15b ),
( "file_type_light_nim", 0xe677 ), // 
( "file_type_light_openHAB", 0xf15b ),
( "file_type_light_pcl", 0xf15b ),
( "file_type_light_pnpm", 0xe71e ), // 
( "file_type_light_prettier", 0xf15b ),
( "file_type_light_prisma", 0xe684 ), // 
( "file_type_light_purescript", 0xe630 ), // 
( "file_type_light_razzle", 0xf15b ),
( "file_type_light_rehype", 0xf15b ),
( "file_type_light_remark", 0xf15b ),
( "file_type_light_retext", 0xf15b ),
( "file_type_light_rubocop", 0xf15b ),
( "file_type_light_shaderlab", 0xf15b ), // 󰷋
( "file_type_light_solidity", 0xf15a ), // 
( "file_type_light_stylelint", 0xe695 ), // 
( "file_type_light_stylus", 0xe759 ), // 
( "file_type_light_symfony", 0xe756 ), // 
( "file_type_light_systemd", 0xf15b ),
( "file_type_light_systemverilog", 0xf15b ),
( "file_type_light_testcafe", 0xf15b ),
( "file_type_light_testjs", 0xf0668 ), // 󰙨
( "file_type_light_tex", 0xe69b ), // 
( "file_type_light_todo", 0xe69c ), // 
( "file_type_light_toml", 0xf0626 ), // 󰘦
( "file_type_light_unibeautify", 0xf15b ),
( "file_type_light_vash", 0xf15b ),
( "file_type_light_vsix", 0xf15b ),
( "file_type_light_vsixmanifest", 0xf15b ),
( "file_type_light_xfl", 0xf15b ),
( "file_type_light_yaml", 0xf0626 ), // 󰘦
( "file_type_light_zeit", 0xf15b ),
( "file_type_lighthouse", 0xf15b ),
( "file_type_lime", 0xf15b ),
( "file_type_lintstagedrc", 0xf15b ),
( "file_type_liquid", 0xf15b ),
( "file_type_lisp", 0xf15b ),
( "file_type_livescript", 0xf15b ),
( "file_type_lnk", 0xf15b ),
( "file_type_locale", 0xf1ab ), // 
( "file_type_log", 0xf1085 ), // 󱂅
( "file_type_lolcode", 0xf15b ),
( "file_type_lsl", 0xf15b ),
( "file_type_lua", 0xe620 ), // 
( "file_type_lync", 0xf15b ),
( "file_type_makefile", 0xe673 ), // 
( "file_type_manifest", 0xf15b ),
( "file_type_manifest_bak", 0xf15b ),
( "file_type_manifest_skip", 0xf15b ),
( "file_type_map", 0xf15b ),
( "file_type_mariadb", 0xe64d ), // 
( "file_type_markdown", 0xf48a ), // 
( "file_type_markdownlint", 0xf48a ), // 
( "file_type_marko", 0xf15b ),
( "file_type_markojs", 0xf15b ),
( "file_type_matlab", 0xf15b ),
( "file_type_maven", 0xe674 ), // 
( "file_type_maxscript", 0xf15b ),
( "file_type_maya", 0xf15b ),
( "file_type_mdx", 0xe73e ), // 
( "file_type_mediawiki", 0xf15b ),
( "file_type_mercurial", 0xf15b ),
( "file_type_meson", 0xf15b ),
( "file_type_meteor", 0xf15b ),
( "file_type_mjml", 0xf15b ),
( "file_type_mlang", 0xf15b ),
( "file_type_mocha", 0xf15b ),
( "file_type_modernizr", 0xe720 ), // 
( "file_type_mojolicious", 0xf15b ),
( "file_type_moleculer", 0xf15b ),
( "file_type_mongo", 0xe7a4 ), // 
( "file_type_monotone", 0xf15b ),
( "file_type_mson", 0xf15b ),
( "file_type_mustache", 0xe228 ), // 
( "file_type_mysql", 0xe704 ), // 
( "file_type_nearly", 0xf15b ),
( "file_type_nest_adapter_js", 0xf031e ),
( "file_type_nest_adapter_ts", 0xf031e ),
( "file_type_nest_controller_js", 0xf031e ),
( "file_type_nest_controller_ts", 0xf031e ),
( "file_type_nest_decorator_js", 0xf031e ),
( "file_type_nest_decorator_ts", 0xf031e ),
( "file_type_nest_filter_js", 0xf031e ),
( "file_type_nest_filter_ts", 0xf031e ),
( "file_type_nest_gateway_js", 0xf031e ),
( "file_type_nest_gateway_ts", 0xf031e ),
( "file_type_nest_guard_js", 0xf031e ),
( "file_type_nest_guard_ts", 0xf031e ),
( "file_type_nest_interceptor_js", 0xf031e ),
( "file_type_nest_interceptor_ts", 0xf031e ),
( "file_type_nest_middleware_js", 0xf031e ),
( "file_type_nest_middleware_ts", 0xf031e ),
( "file_type_nest_module_js", 0xf031e ),
( "file_type_nest_module_ts", 0xf031e ),
( "file_type_nest_pipe_js", 0xf031e ),
( "file_type_nest_pipe_ts", 0xf031e ),
( "file_type_nest_service_js", 0xf031e ),
( "file_type_nest_service_ts", 0xf031e ),
( "file_type_nestjs", 0xf031e ),
( "file_type_netlify", 0xf15b ),
( "file_type_next", 0xf15b ),
( "file_type_ng_component_css", 0xe749 ), // 
( "file_type_ng_component_dart", 0xe64c ), // 
( "file_type_ng_component_html", 0xe736 ), // 
( "file_type_ng_component_js", 0xe781 ), // 
( "file_type_ng_component_js2", 0xe781 ), // 
( "file_type_ng_component_less", 0xe758 ), // 
( "file_type_ng_component_sass", 0xe74b ), // 
( "file_type_ng_component_scss", 0xe749 ), // 
( "file_type_ng_component_ts", 0xf06e6 ), // 󰛦
( "file_type_ng_component_ts2", 0xf06e6 ), // 󰛦
( "file_type_ng_controller_js", 0xe781 ), // ..
( "file_type_ng_controller_ts", 0xf06e6 ),
( "file_type_ng_directive_dart", 0xe64c ),
( "file_type_ng_directive_js", 0xe781 ),
( "file_type_ng_directive_js2", 0xe781 ),
( "file_type_ng_directive_ts", 0xf06e6 ),
( "file_type_ng_directive_ts2", 0xf06e6 ),
( "file_type_ng_guard_dart", 0xe64c ),
( "file_type_ng_guard_js", 0xe781 ),
( "file_type_ng_guard_ts", 0xf06e6 ), // ..
( "file_type_ng_interceptor_dart", 0xe64c ),
( "file_type_ng_interceptor_js", 0xe781 ),
( "file_type_ng_interceptor_ts", 0xf06e6 ),
( "file_type_ng_module_dart", 0xe64c ),
( "file_type_ng_module_js", 0xe781 ),
( "file_type_ng_module_js2", 0xe781 ),
( "file_type_ng_module_ts", 0xf06e6 ),
( "file_type_ng_module_ts2", 0xf06e6 ),
( "file_type_ng_pipe_dart", 0xe64c ),
( "file_type_ng_pipe_js", 0xe781 ),
( "file_type_ng_pipe_js2", 0xe781 ),
( "file_type_ng_pipe_ts", 0xf06e6 ),
( "file_type_ng_pipe_ts2", 0xf06e6 ),
( "file_type_ng_routing_dart", 0xe64c ),
( "file_type_ng_routing_js", 0xe781 ),
( "file_type_ng_routing_js2", 0xe781 ),
( "file_type_ng_routing_ts", 0xf06e6 ),
( "file_type_ng_routing_ts2", 0xe781 ),
( "file_type_ng_service_dart", 0xe64c ),
( "file_type_ng_service_js", 0xe781 ),
( "file_type_ng_service_js2", 0xe781 ),
( "file_type_ng_service_ts", 0xf06e6 ),
( "file_type_ng_service_ts2", 0xf06e6 ),
( "file_type_ng_smart_component_dart", 0xe64c ),
( "file_type_ng_smart_component_js", 0xe781 ),
( "file_type_ng_smart_component_js2", 0xe781 ),
( "file_type_ng_smart_component_ts", 0xf06e6 ),
( "file_type_ng_smart_component_ts2", 0xf06e6 ),
( "file_type_ng_tailwind", 0xf13ff ), // 󱏿
( "file_type_nginx", 0xe776 ), // 
( "file_type_nim", 0xe677 ), // 
( "file_type_nimble", 0xf15b ),
( "file_type_ninja", 0xf0774 ), // 󰝴
( "file_type_nix", 0xf1105 ), // 󱄅
( "file_type_njsproj", 0xf15b ),
( "file_type_node", 0xf0399 ), // 󰎙
( "file_type_node2", 0xf0399 ), // 󰎙
( "file_type_nodemon", 0xf0399 ), // 󰎙
( "file_type_npm", 0xe71e ), // 
( "file_type_nsi", 0xf15b ),
( "file_type_nsri", 0xf15b ),
( "file_type_nsri-integrity", 0xf15b ),
( "file_type_nuget", 0xf15b ),
( "file_type_numpy", 0xf15b ),
( "file_type_nunjucks", 0xe679 ), // 
( "file_type_nushell", 0xf07c6 ), // 󰟆
( "file_type_nuxt", 0xf1106 ), // 󱄆
( "file_type_nyc", 0xf15b ),
( "file_type_objectivec", 0xf0bf3 ), // 󰯳
( "file_type_objectivecpp", 0xf0bf3 ), // 󰯳
( "file_type_ocaml", 0xe67a ), // 
( "file_type_onenote", 0xf15b ),
( "file_type_openHAB", 0xf15b ),
( "file_type_opencl", 0xf15b ),
( "file_type_org", 0xe633 ), // 
( "file_type_outlook", 0xf0d22 ), // 󰴢
( "file_type_ovpn", 0xf15b ), //
( "file_type_package", 0xeb29 ), // 
( "file_type_paket", 0xf03d7 ), // 󰏗
( "file_type_patch", 0xf15b ), //
( "file_type_pcl", 0xf15b ), //
( "file_type_pddl", 0xf15b),
( "file_type_pddl_happenings", 0xf1b5 ), //
( "file_type_pddl_plan", 0xf15b ),
( "file_type_pdf", 0xeaeb ), // 
( "file_type_pdf2", 0xe67d ), // 
( "file_type_perl", 0xe769 ), // 
( "file_type_perl2", 0xe769 ),
( "file_type_perl6", 0xe769 ),
( "file_type_pgsql", 0xe76e ), // 
( "file_type_photoshop", 0xe7b8 ), // 
( "file_type_photoshop2", 0xe7b8 ),
( "file_type_php", 0xe73d ), // 
( "file_type_php2", 0xe608 ), // 
( "file_type_php3", 0xf031f ), // 󰌟
( "file_type_phpcsfixer", 0xe73d ),
( "file_type_phpunit", 0xe73d ),
( "file_type_phraseapp", 0xf15b ),
( "file_type_pine", 0xf15b ),
( "file_type_pip", 0xe73c ), // 
( "file_type_plantuml", 0xf15b ),
( "file_type_platformio", 0xf15b ),
( "file_type_plsql", 0xf15b ),
( "file_type_plsql_package", 0xf15b ),
( "file_type_plsql_package_body", 0xf15b ),
( "file_type_plsql_package_header", 0xf15b ),
( "file_type_plsql_package_spec", 0xf15b ),
( "file_type_pnpm", 0xe71e ), // 
( "file_type_poedit", 0xf15b ),
( "file_type_polymer", 0xf0421 ), // 󰐡
( "file_type_pony", 0xf15b ),
( "file_type_postcss", 0xf13c ), // 
( "file_type_postcssconfig", 0xf13c ),
( "file_type_powerpoint", 0xf0227 ), // 󰈧
( "file_type_powerpoint2", 0xf0227 ),
( "file_type_powershell", 0xf0a0a ),
( "file_type_powershell2", 0xf0a0a ),
( "file_type_powershell_format", 0xf0a0a ),
( "file_type_powershell_psd", 0xf0a0a ),
( "file_type_powershell_psd2", 0xf0a0a ),
( "file_type_powershell_psm", 0xf0a0a ),
( "file_type_powershell_psm2", 0xf0a0a ),
( "file_type_powershell_types", 0xf0a0a ), // 󰨊
( "file_type_precommit", 0xe729 ), // 
( "file_type_prettier", 0xf15b ),
( "file_type_prisma", 0xe684 ), // 
( "file_type_processinglang", 0xf15b ),
( "file_type_procfile", 0xf15b ),
( "file_type_progress", 0xf15b ),
( "file_type_prolog", 0xf15b ),
( "file_type_prometheus", 0xf15b ),
( "file_type_protobuf", 0xf15b ),
( "file_type_protractor", 0xf15b ),
( "file_type_publisher", 0xf15b ),
( "file_type_pug", 0xf15b ),
( "file_type_puppet", 0xe631 ), // 
( "file_type_purescript", 0xe630 ), // 
( "file_type_pyret", 0xf15b ),
( "file_type_python", 0xe73c ), // 
( "file_type_pyup", 0xe73c ),
( "file_type_q", 0xf15b ),
( "file_type_qbs", 0xf15b ),
( "file_type_qlikview", 0xf15b ),
( "file_type_qml", 0xf15b ),
( "file_type_qmldir", 0xf15b ),
( "file_type_qsharp", 0xf15b ),
( "file_type_quasar", 0xf15b ),
( "file_type_r", 0xf0b19 ), // 󰬙
( "file_type_racket", 0xf15b ),
( "file_type_rails", 0xe604 ), // 
( "file_type_rake", 0xf15b ),
( "file_type_raml", 0xf0626 ), // 󰘦
( "file_type_razor", 0xf1997 ), // 󱦗
( "file_type_razzle", 0xf15b ),
( "file_type_reactjs", 0xe7ba ), // 
( "file_type_reacttemplate", 0xe7ba), // 
( "file_type_reactts", 0xe7ba ), // 
( "file_type_reason", 0xe687 ), // 
( "file_type_red", 0xf15b ),
( "file_type_registry", 0xf15b ),
( "file_type_rego", 0xf15b ),
( "file_type_rehype", 0xf15b ),
( "file_type_remark", 0xf15b ),
( "file_type_renovate", 0xf15b),
( "file_type_rescript", 0xf15b ),
( "file_type_rest", 0xf15b),
( "file_type_retext", 0xf15b),
( "file_type_rexx", 0xf15b ),
( "file_type_riot", 0xf15b ),
( "file_type_rmd", 0xf15b ),
( "file_type_robotframework", 0xf15b),
( "file_type_robots", 0xf06a9 ), // 󰚩
( "file_type_rollup", 0xe689 ), // 
( "file_type_rproj", 0xf15b ),
( "file_type_rspec", 0xf15b ),
( "file_type_rubocop", 0xf15b ),
( "file_type_ruby", 0xf0d2d ), // 󰴭
( "file_type_rust", 0xe7a8 ), // 
( "file_type_rust_toolchain", 0xe7a8 ),
( "file_type_sails", 0xf15b ),
( "file_type_saltstack", 0xf15b ),
( "file_type_san", 0xf15b ),
( "file_type_sas", 0xe74b  ),
( "file_type_sass", 0xe74b ), // 
( "file_type_sbt", 0xe68d ), // 
( "file_type_scala", 0xe737 ), // 
( "file_type_scilab", 0xf15b ),
( "file_type_script", 0xf06e6 ),
( "file_type_scss", 0xf13c ),
( "file_type_scss2", 0xf13c ),
( "file_type_sdlang", 0xf15b),
( "file_type_sentry", 0xf15b ),
( "file_type_sequelize", 0xf15b ),
( "file_type_serverless", 0xf15b ),
( "file_type_shaderlab", 0xf15b ),
( "file_type_shell", 0xe691 ), // 
( "file_type_silverstripe", 0xf15b ),
( "file_type_sketch", 0xf15b ),
( "file_type_skipper", 0xf15b ),
( "file_type_slang", 0xf15b ),
( "file_type_slice", 0xf15b ),
( "file_type_slim", 0xf15b ),
( "file_type_sln", 0xf15b ),
( "file_type_sln2", 0xf15b ),
( "file_type_smarty", 0xf15b ),
( "file_type_snapcraft", 0xf15b ),
( "file_type_snort", 0xf15b ),
( "file_type_snyk", 0xf15b ),
( "file_type_solidarity", 0xf15b ),
( "file_type_solidity", 0xf15b ),
( "file_type_source", 0xf15b ),
( "file_type_spacengine", 0xf15b ),
( "file_type_sqf", 0xf15b ),
( "file_type_sql", 0xf0b86 ), // 󰮆
( "file_type_sqlite", 0xe7c4 ), // 
( "file_type_squirrel", 0xf15b ),
( "file_type_sss", 0xf15b ),
( "file_type_stan", 0xf15b ),
( "file_type_stata", 0xf15b ),
( "file_type_stencil", 0xf15b ),
( "file_type_storyboard", 0xf15b ),
( "file_type_storybook", 0xf15b ),
( "file_type_stylable", 0xf15b ),
( "file_type_style", 0xf15b ),
( "file_type_styled", 0xf15b ),
( "file_type_stylelint", 0xf15b ),
( "file_type_stylish_haskell", 0xf15b ),
( "file_type_stylus", 0xf15b ),
( "file_type_subversion", 0xf15b ),
( "file_type_svelte", 0xe697 ), // 
( "file_type_svg", 0xe698 ), // 
( "file_type_swagger", 0xf15b ),
( "file_type_swift", 0xe755 ), // 
( "file_type_swig", 0xf15b ),
( "file_type_symfony", 0xe756 ), // 
( "file_type_systemd", 0xf15b ),
( "file_type_systemverilog", 0xf15b ),
( "file_type_t4tt", 0xf15b ),
( "file_type_tailwind", 0xf13ff ), // 󱏿
( "file_type_tcl", 0xf15b ),
( "file_type_tera", 0xf15b ),
( "file_type_terraform", 0xf1062 ), // 󱁢
( "file_type_test", 0xf0668 ),
( "file_type_testcafe", 0xf15b ),
( "file_type_testjs", 0xf0668 ), // 󰙨
( "file_type_testts", 0xf0668 ),
( "file_type_tex", 0xe69b ), // 
( "file_type_text", 0xf09a8 ), // 󰦨
( "file_type_textile", 0xf15b ),
( "file_type_tfs", 0xf15b ),
( "file_type_todo", 0xe69c ), // 
( "file_type_toml", 0xf0626 ), // 󰘦
( "file_type_tox", 0xf15b ),
( "file_type_travis", 0xe77e ), // 
( "file_type_tsconfig", 0xe628 ), // 
( "file_type_tslint", 0xe628 ), // 
( "file_type_tt", 0xf15b ),
( "file_type_ttcn", 0xf15b ),
( "file_type_twig", 0xe61c ), // 
( "file_type_typescript", 0xf06e6 ), // 󰛦
( "file_type_typescript_official", 0xf06e6 ), // 󰛦
( "file_type_typescriptdef", 0xf06e6 ), // 󰛦
( "file_type_typescriptdef_official", 0xf06e6 ), // 󰛦
( "file_type_typo3", 0xe772 ), // 
( "file_type_unibeautify", 0xf15b ),
( "file_type_vagrant", 0xf15b ),
( "file_type_vala", 0xe69e ), // 
( "file_type_vapi", 0xf15b ),
( "file_type_vash", 0xf15b ),
( "file_type_vb", 0xf15b ),
( "file_type_vba", 0xf15b ),
( "file_type_vbhtml", 0xf15b ),
( "file_type_vbproj", 0xf15b ),
( "file_type_vcxproj", 0xf15b ),
( "file_type_velocity", 0xf15b ),
( "file_type_verilog", 0xf15b ),
( "file_type_version", 0xeb78 ), // 
( "file_type_vhdl", 0xf15b ),
( "file_type_video", 0xf022b ), // 󰈫
( "file_type_view", 0xf15b ),
( "file_type_vim", 0xe62b ), // 
( "file_type_vlang", 0xe6ac ), // 
( "file_type_volt", 0xf15b ),
( "file_type_vscode", 0xe70c ), // 
( "file_type_vscode-insiders", 0xe70c ), // 
( "file_type_vscode2", 0xe70c ), // 
( "file_type_vscode3", 0xe70c ), // 
( "file_type_vsix", 0xf15b ),
( "file_type_vsixmanifest", 0xf15b ),
( "file_type_vue", 0xe6a0), // 
( "file_type_vueconfig", 0xe6a0 ), // 
( "file_type_wallaby", 0xf15b ),
( "file_type_wasm", 0xe6a1 ), // 
( "file_type_watchmanconfig", 0xf15b ),
( "file_type_webp", 0xf021f ), // 󰈟
( "file_type_webpack", 0xf072b ), // 󰜫
( "file_type_wenyan", 0xf15b ),
( "file_type_wercker", 0xf15b ),
( "file_type_wolfram", 0xf0a9a ), // 󰪚
( "file_type_word", 0xe6a5 ), // 
( "file_type_word2", 0xe6a5 ), // 
( "file_type_wpml", 0xf15b ),
( "file_type_wurst", 0xf15b ),
( "file_type_wxml", 0xf15b ),
( "file_type_wxss", 0xf15b ),
( "file_type_xcode", 0xf15b ),
( "file_type_xfl", 0xf15b ),
( "file_type_xib", 0xf15b ),
( "file_type_xliff", 0xf15b ),
( "file_type_xmake", 0xf15b ),
( "file_type_xml", 0xf05c0 ), // 󰗀
( "file_type_xquery", 0xf15b ),
( "file_type_xsl", 0xf15b ),
( "file_type_yacc", 0xf15b ),
( "file_type_yaml", 0xf0626 ), // 󰘦
( "file_type_yamllint", 0xf0626 ), // 󰘦
( "file_type_yandex", 0xf15b ),
( "file_type_yang", 0xf15b ),
( "file_type_yarn", 0xe6a7 ), // 
( "file_type_yeoman", 0xf15b ),
( "file_type_zeit", 0xf15b ),
( "file_type_zig", 0xe6a9 ), // 
( "file_type_zip", 0xf1c6 ), // 
( "file_type_zip2", 0xf1c6 ),// 
]
