// SEE ./README on how to edit this file
[
    ( "adapter.js"                 , "file_type_nest_adapter_js"          ),
    ( "adapter.ts"                 , "file_type_nest_adapter_ts"          ),
    ( "component.dart"             , "file_type_ng_component_dart"        ),
    ( "component.js"               , "file_type_ng_component_js2"         ),
    ( "component.ts"               , "file_type_ng_component_ts2"         ),
    ( "container.dart"             , "file_type_ng_smart_component_dart"  ),
    ( "container.js"               , "file_type_ng_smart_component_js2"   ),
    ( "container.ts"               , "file_type_ng_smart_component_ts2"   ),
    ( "controller.js"              , "file_type_nest_controller_js"       ),
    ( "controller.ts"              , "file_type_nest_controller_ts"       ),
    ( "css.map"                    , "file_type_cssmap"                   ),
    ( "d.ts"                       , "file_type_typescriptdef_official"   ),
    ( "decorator.js"               , "file_type_nest_decorator_js"        ),
    ( "decorator.ts"               , "file_type_nest_decorator_ts"        ),
    ( "directive.dart"             , "file_type_ng_directive_dart"        ),
    ( "directive.js"               , "file_type_ng_directive_js2"         ),
    ( "directive.ts"               , "file_type_ng_directive_ts2"         ),
    ( "eslintrc.js"                , "file_type_eslint"                   ),
    ( "filter.js"                  , "file_type_nest_filter_js"           ),
    ( "filter.ts"                  , "file_type_nest_filter_ts"           ),
    ( "gateway.js"                 , "file_type_nest_gateway_js"          ),
    ( "gateway.ts"                 , "file_type_nest_gateway_ts"          ),
    ( "gradle.kts"                 , "file_type_gradle"                   ),
    ( "guard.js"                   , "file_type_nest_guard_js"            ),
    ( "guard.ts"                   , "file_type_nest_guard_ts"            ),
    ( "interceptor.dart"           , "file_type_ng_interceptor_dart"      ),
    ( "interceptor.js"             , "file_type_nest_interceptor_js"      ),
    ( "interceptor.ts"             , "file_type_nest_interceptor_ts"      ),
    ( "js.flow"                    , "file_type_flow"                     ),
    ( "js.map"                     , "file_type_jsmap"                    ),
    ( "js.snap"                    , "file_type_jest_snapshot"            ),
    ( "middleware.js"              , "file_type_nest_middleware_js"       ),
    ( "middleware.ts"              , "file_type_nest_middleware_ts"       ),
    ( "module.js"                  , "file_type_nest_module_js"           ),
    ( "module.ts"                  , "file_type_nest_module_ts"           ),
    ( "page.dart"                  , "file_type_ng_smart_component_dart"  ),
    ( "page.js"                    , "file_type_ng_smart_component_js2"   ),
    ( "page.ts"                    , "file_type_ng_smart_component_ts2"   ),
    ( "pipe.dart"                  , "file_type_ng_pipe_dart"             ),
    ( "pipe.js"                    , "file_type_nest_pipe_js"             ),
    ( "pipe.ts"                    , "file_type_nest_pipe_ts"             ),
    ( "routing.dart"               , "file_type_ng_routing_dart"          ),
    ( "routing.js"                 , "file_type_ng_routing_js2"           ),
    ( "routing.ts"                 , "file_type_ng_routing_ts2"           ),
    ( "service.dart"               , "file_type_ng_service_dart"          ),
    ( "service.js"                 , "file_type_nest_service_js"          ),
    ( "service.ts"                 , "file_type_nest_service_ts"          ),
    ( "tar.gz"                     , "file_type_zip"                      ),
    ( "tar.xz"                     , "file_type_zip"                      ),
    ( "tar.zst"                    , "file_type_zip"                      ),
]
