// SEE ./README on how to edit this file
[
( "3g2"                     , "file_type_video"                   ),
( "3gp"                     , "file_type_video"                   ),
( "7z"                      , "file_type_zip2"                    ),
( "P"                       , "file_type_prolog"                  ),
( "a"                       , "file_type_binary"                  ),
( "aac"                     , "file_type_audio"                   ),
( "accda"                   , "file_type_access2"                 ),
( "accdb"                   , "file_type_access"                  ),
( "accdc"                   , "file_type_access2"                 ),
( "accde"                   , "file_type_access2"                 ),
( "accdp"                   , "file_type_access2"                 ),
( "accdr"                   , "file_type_access2"                 ),
( "accdt"                   , "file_type_access2"                 ),
( "accdu"                   , "file_type_access2"                 ),
( "act"                     , "file_type_audio"                   ),
( "ade"                     , "file_type_access2"                 ),
( "adp"                     , "file_type_access2"                 ),
( "afdesign"                , "file_type_affinitydesigner"        ),
( "affinitydesigner"        , "file_type_affinitydesigner"        ),
( "affinityphoto"           , "file_type_affinityphoto"           ),
( "affinitypublisher"       , "file_type_affinitypublisher"       ),
( "afphoto"                 , "file_type_affinityphoto"           ),
( "afpub"                   , "file_type_affinitypublisher"       ),
( "ai"                      , "file_type_ai2"                     ),
( "aiff"                    , "file_type_audio"                   ),
( "amr"                     , "file_type_audio"                   ),
( "amv"                     , "file_type_video"                   ),
( "ape"                     , "file_type_audio"                   ),
( "app"                     , "file_type_binary"                  ),
( "ascx"                    , "file_type_aspx"                    ),
( "asf"                     , "file_type_video"                   ),
( "aspx"                    , "file_type_aspx"                    ),
( "au"                      , "file_type_audio"                   ),
( "avi"                     , "file_type_video"                   ),
( "avif"                    , "file_type_avif"                    ),
( "awk"                     , "file_type_awk"                     ),
( "babelignore"             , "file_type_babel2"                  ),
( "babelrc"                 , "file_type_babel2"                  ),
( "bazel"                   , "file_type_bazel"                   ),
( "bazelrc"                 , "file_type_bazel"                   ),
( "bb"                      , "file_type_blitzbasic"              ),
( "bcmx"                    , "file_type_outlook"                 ),
( "bin"                     , "file_type_binary"                  ),
( "bithoundrc"              , "file_type_bithound"                ),
( "bmp"                     , "file_type_image"                   ),
( "boringignore"            , "file_type_darcs"                   ),
( "bowerrc"                 , "file_type_bower"                   ),
( "browserslistrc"          , "file_type_browserslist"            ),
( "buckconfig"              , "file_type_buckbuild"               ),
( "bz"                      , "file_type_zip2"                    ),
( "bz2"                     , "file_type_zip2"                    ),
( "bzip2"                   , "file_type_zip2"                    ),
( "bzrignore"               , "file_type_bazaar"                  ),
( "c"                       , "file_type_c"                       ),
( "cake"                    , "file_type_cake"                    ),
( "cargo-lock"              , "file_type_rust"                    ),
( "cc"                      , "file_type_cpp"                     ),
( "cer"                     , "file_type_cert"                    ),
( "cfignore"                , "file_type_cloudfoundry"            ),
( "cjm"                     , "file_type_clojure"                 ),
( "cl"                      , "file_type_opencl"                  ),
( "class"                   , "file_type_class"                   ),
( "cljc"                    , "file_type_clojure"                 ),
( "cljs"                    , "file_type_clojurescript"           ),
( "cma"                     , "file_type_binary"                  ),
( "cmi"                     , "file_type_binary"                  ),
( "cmo"                     , "file_type_binary"                  ),
( "cmx"                     , "file_type_binary"                  ),
( "cmxa"                    , "file_type_binary"                  ),
( "coffeelintignore"        , "file_type_coffeelint"              ),
( "condarc"                 , "file_type_conda"                   ),
( "cpp"                     , "file_type_cpp"                     ),
( "crec"                    , "file_type_lync"                    ),
( "crl"                     , "file_type_cert"                    ),
( "crt"                     , "file_type_cert"                    ),
( "cs"                      , "file_type_csharp"                  ),
( "csproj"                  , "file_type_csproj"                  ),
( "csr"                     , "file_type_cert"                    ),
( "css"                     , "file_type_css"                     ),
( "csslintrc"               , "file_type_csslint"                 ),
( "csv"                     , "file_type_csv"                     ),
( "csx"                     , "file_type_csharp2"                 ),
( "cvsignore"               , "file_type_cvs"                     ),
( "db"                      , "file_type_db"                      ),
( "db3"                     , "file_type_sqlite"                  ),
( "dct"                     , "file_type_audio"                   ),
( "der"                     , "file_type_cert"                    ),
( "dio"                     , "file_type_drawio"                  ),
( "divx"                    , "file_type_video"                   ),
( "djt"                     , "file_type_django"                  ),
( "dll"                     , "file_type_binary"                  ),
( "doc"                     , "file_type_word2"                   ),
( "docm"                    , "file_type_word2"                   ),
( "docx"                    , "file_type_word2"                   ),
( "doczrc"                  , "file_type_docz"                    ),
( "dojorc"                  , "file_type_dojo"                    ),
( "dot"                     , "file_type_word2"                   ),
( "dotm"                    , "file_type_word2"                   ),
( "dotx"                    , "file_type_word2"                   ),
( "drawio"                  , "file_type_drawio"                  ),
( "dss"                     , "file_type_audio"                   ),
( "dta"                     , "file_type_stata"                   ),
( "dvc"                     , "file_type_dvc"                     ),
( "dvf"                     , "file_type_audio"                   ),
( "eco"                     , "file_type_docpad"                  ),
( "editorconfig"            , "file_type_editorconfig"            ),
( "ejs"                     , "file_type_ejs"                     ),
( "el"                      , "file_type_emacs"                   ),
( "elc"                     , "file_type_emacs"                   ),
( "elm"                     , "file_type_elm"                     ),
( "ember-cli"               , "file_type_ember"                   ),
( "enc"                     , "file_type_license"                 ),
( "ensime"                  , "file_type_ensime"                  ),
( "env"                     , "file_type_env"                     ),
( "eot"                     , "file_type_font"                    ),
( "eps"                     , "file_type_eps"                     ),
( "eskip"                   , "file_type_skipper"                 ),
( "eslintcache"             , "file_type_eslint2"                 ),
( "eslintignore"            , "file_type_eslint2"                 ),
( "eslintrc"                , "file_type_eslint2"                 ),
( "exe"                     , "file_type_binary"                  ),
( "exp"                     , "file_type_tcl"                     ),
( "f4a"                     , "file_type_video"                   ),
( "f4b"                     , "file_type_video"                   ),
( "f4p"                     , "file_type_video"                   ),
( "f4v"                     , "file_type_video"                   ),
( "fbx"                     , "file_type_fbx"                     ),
( "fnl"                     , "file_type_fennel"                  ),
( "fig"                     , "file_type_matlab"                  ),
( "firebaserc"              , "file_type_firebase"                ),
( "fish"                    , "file_type_shell"                   ),
( "fla"                     , "file_type_fla"                     ),
( "flac"                    , "file_type_audio"                   ),
( "flooignore"              , "file_type_floobits"                ),
( "flowconfig"              , "file_type_flow"                    ),
( "flutter-plugins"         , "file_type_flutter"                 ),
( "flv"                     , "file_type_video"                   ),
( "fods"                    , "file_type_excel2"                  ),
( "fossaignore"             , "file_type_fossa"                   ),
( "fs"                      , "file_type_fsharp"                  ),
( "fsproj"                  , "file_type_fsproj"                  ),
( "gemfile"                 , "file_type_bundler"                 ),
( "gif"                     , "file_type_image"                   ),
( "gitattributes"           , "file_type_git"                     ),
( "gitconfig"               , "file_type_git"                     ),
( "gitignore"               , "file_type_git"                     ),
( "gitkeep"                 , "file_type_git"                     ),
( "gitmodules"              , "file_type_git"                     ),
( "gmx"                     , "file_type_gamemaker"               ),
( "go"                      , "file_type_go"                      ),
( "gqlconfig"               , "file_type_graphql"                 ),
( "gradle"                  , "file_type_gradle2"                 ),
( "graphqlconfig"           , "file_type_graphql_config"          ),
( "gsm"                     , "file_type_audio"                   ),
( "gvimrc"                  , "file_type_vim"                     ),
( "gz"                      , "file_type_zip2"                    ),
( "h"                       , "file_type_cheader"                 ),
( "hgignore"                , "file_type_mercurial"               ),
( "hl"                      , "file_type_binary"                  ),
( "hpp"                     , "file_type_cppheader"               ),
( "hs"                      , "file_type_haskell"                 ),
( "html"                    , "file_type_html"                    ),
( "htmlhintrc"              , "file_type_htmlhint"                ),
( "huskyrc"                 , "file_type_husky"                   ),
( "hxp"                     , "file_type_lime"                    ),
( "hxproj"                  , "file_type_haxedevelop"             ),
( "ibc"                     , "file_type_idrisbin"                ),
( "ico"                     , "file_type_image"                   ),
( "idr"                     , "file_type_idris"                   ),
( "ignore-glob"             , "file_type_fossil"                  ),
( "iklax"                   , "file_type_audio"                   ),
( "ilk"                     , "file_type_binary"                  ),
( "inc"                     , "file_type_inc"                     ),
( "include"                 , "file_type_inc"                     ),
( "infopathxml"             , "file_type_infopath"                ),
( "ino"                     , "file_type_arduino"                 ),
( "ipkg"                    , "file_type_idrispkg"                ),
( "ipynb"                   , "file_type_jupyter"                 ),
( "iuml"                    , "file_type_plantuml"                ),
( "ivs"                     , "file_type_audio"                   ),
( "jade-lintrc"             , "file_type_pug"                     ),
( "jar"                     , "file_type_jar"                     ),
( "java"                    , "file_type_java"                    ),
( "jestrc"                  , "file_type_jest"                    ),
( "jpeg"                    , "file_type_image"                   ),
( "jpg"                     , "file_type_image"                   ),
( "jpmignore"               , "file_type_jpm"                     ),
( "js"                      , "file_type_js"                      ),
( "jsbeautify"              , "file_type_jsbeautify"              ),
( "jsbeautifyrc"            , "file_type_jsbeautify"              ),
( "jshintignore"            , "file_type_jshint"                  ),
( "jshintrc"                , "file_type_jshint"                  ),
( "json"                    , "file_type_json_official"           ),
( "json-ld"                 , "file_type_jsonld"                  ),
( "json5"                   , "file_type_json5"                   ),
( "jsonld"                  , "file_type_jsonld"                  ),
( "jsp"                     , "file_type_jsp"                     ),
( "jss"                     , "file_type_jss"                     ),
( "jl"                      , "file_type_julia"                   ),
( "kdl"                     , "file_type_config"                  ),
( "key"                     , "file_type_key"                     ),
( "kit"                     , "file_type_codekit"                 ),
( "kiteignore"              , "file_type_kite"                    ),
( "kt"                      , "file_type_kotlin"                  ),
( "laccdb"                  , "file_type_access2"                 ),
( "ldb"                     , "file_type_access2"                 ),
( "lib"                     , "file_type_binary"                  ),
( "licence"                 , "file_type_license"                 ),
( "lidr"                    , "file_type_idris"                   ),
( "lintstagedrc"            , "file_type_lintstagedrc"            ),
( "liquid"                  , "file_type_liquid"                  ),
( "lnk"                     , "file_type_lnk"                     ),
( "lock"                    , "emoji_type_lock"                   ),
( "log"                     , "file_type_log"                     ),
( "ls"                      , "file_type_livescript"              ),
( "lua"                     , "file_type_lua"                     ),
( "lucee"                   , "file_type_cf2"                     ),
( "m2v"                     , "file_type_video"                   ),
( "m4a"                     , "file_type_audio"                   ),
( "m4b"                     , "file_type_audio"                   ),
( "m4p"                     , "file_type_audio"                   ),
( "m4v"                     , "file_type_video"                   ),
( "mailmap"                 , "file_type_git"                     ),
( "mam"                     , "file_type_access2"                 ),
( "manifest"                , "file_type_manifest"                ),
( "map"                     , "file_type_map"                     ),
( "maq"                     , "file_type_access2"                 ),
( "markdown"                , "file_type_markdown"                ),
( "master"                  , "file_type_layout"                  ),
( "md"                      , "file_type_markdown"                ),
( "mdb"                     , "file_type_access2"                 ),
( "mdown"                   , "file_type_markdown"                ),
( "mdw"                     , "file_type_access2"                 ),
( "mdx"                     , "file_type_mdx"                     ),
( "merlin"                  , "file_type_ocaml"                   ),
( "metadata"                , "file_type_flutter"                 ),
( "mex"                     , "file_type_matlab"                  ),
( "mexn"                    , "file_type_matlab"                  ),
( "mexrs6"                  , "file_type_matlab"                  ),
( "mk3d"                    , "file_type_video"                   ),
( "mkv"                     , "file_type_video"                   ),
( "mmf"                     , "file_type_audio"                   ),
( "mn"                      , "file_type_matlab"                  ),
( "mo"                      , "file_type_poedit"                  ),
( "modernizr"               , "file_type_modernizr"               ),
( "mogg"                    , "file_type_audio"                   ),
( "mov"                     , "file_type_video"                   ),
( "mp2"                     , "file_type_video"                   ),
( "mp3"                     , "file_type_audio"                   ),
( "mp4"                     , "file_type_video"                   ),
( "mpc"                     , "file_type_audio"                   ),
( "mpe"                     , "file_type_video"                   ),
( "mpeg"                    , "file_type_video"                   ),
( "mpeg2"                   , "file_type_video"                   ),
( "mpg"                     , "file_type_video"                   ),
( "mpv"                     , "file_type_video"                   ),
( "msg"                     , "file_type_outlook"                 ),
( "mst"                     , "file_type_mustache"                ),
( "msv"                     , "file_type_audio"                   ),
( "mtn-ignore"              , "file_type_monotone"                ),
( "mum"                     , "file_type_matlab"                  ),
( "mustache"                , "file_type_mustache"                ),
( "mx"                      , "file_type_matlab"                  ),
( "mx3"                     , "file_type_matlab"                  ),
( "mysql"                   , "file_type_mysql"                   ),
( "n"                       , "file_type_binary"                  ),
( "ndll"                    , "file_type_binary"                  ),
( "nix"                     , "file_type_nix"                     ),
( "njs"                     , "file_type_nunjucks"                ),
( "njsproj"                 , "file_type_njsproj"                 ),
( "node-version"            , "file_type_node2"                   ),
( "nowignore"               , "file_type_zeit"                    ),
( "npmignore"               , "file_type_npm"                     ),
( "npmrc"                   , "file_type_npm"                     ),
( "npy"                     , "file_type_numpy"                   ),
( "npz"                     , "file_type_numpy"                   ),
( "nsriignore"              , "file_type_nsri"                    ),
( "nsrirc"                  , "file_type_nsri"                    ),
( "nsv"                     , "file_type_video"                   ),
( "nu"                      , "file_type_nushell"                 ),
( "nunj"                    , "file_type_nunjucks"                ),
( "nupkg"                   , "file_type_nuget"                   ),
( "nuspec"                  , "file_type_nuget"                   ),
( "nvmrc"                   , "file_type_node2"                   ),
( "nycrc"                   , "file_type_nyc"                     ),
( "o"                       , "file_type_binary"                  ),
( "obj"                     , "file_type_binary"                  ),
( "ocrec"                   , "file_type_lync"                    ),
( "ods"                     , "file_type_excel2"                  ),
( "oft"                     , "file_type_outlook"                 ),
( "oga"                     , "file_type_audio"                   ),
( "ogg"                     , "file_type_audio"                   ),
( "ogv"                     , "file_type_video"                   ),
( "one"                     , "file_type_onenote"                 ),
( "onepkg"                  , "file_type_onenote"                 ),
( "onetoc"                  , "file_type_onenote"                 ),
( "onetoc2"                 , "file_type_onenote"                 ),
( "opencl"                  , "file_type_opencl"                  ),
( "opus"                    , "file_type_audio"                   ),
( "org"                     , "file_type_org"                     ),
( "otf"                     , "file_type_font"                    ),
( "otm"                     , "file_type_outlook"                 ),
( "ovpn"                    , "file_type_ovpn"                    ),
( "p12"                     , "file_type_cert"                    ),
( "p4ignore"                , "file_type_helix"                   ),
( "p7b"                     , "file_type_cert"                    ),
( "p7r"                     , "file_type_cert"                    ),
( "pa"                      , "file_type_powerpoint2"             ),
( "packages"                , "file_type_flutter_package"         ),
( "patch"                   , "file_type_patch"                   ),
( "pcd"                     , "file_type_pcl"                     ),
( "pck"                     , "file_type_plsql_package"           ),
( "pdb"                     , "file_type_binary"                  ),
( "pde"                     , "file_type_arduino"                 ),
( "pdf"                     , "file_type_pdf2"                    ),
( "pem"                     , "file_type_key"                     ),
( "pex"                     , "file_type_xml"                     ),
( "pfa"                     , "file_type_font"                    ),
( "pfb"                     , "file_type_font"                    ),
( "pfx"                     , "file_type_cert"                    ),
( "pgsql"                   , "file_type_pgsql"                   ),
( "phar"                    , "file_type_php3"                    ),
( "php"                     , "file_type_php"                     ),
( "php1"                    , "file_type_php3"                    ),
( "php2"                    , "file_type_php3"                    ),
( "php3"                    , "file_type_php3"                    ),
( "php4"                    , "file_type_php3"                    ),
( "php5"                    , "file_type_php3"                    ),
( "php6"                    , "file_type_php3"                    ),
( "php_cs"                  , "file_type_phpcsfixer"              ),
( "phps"                    , "file_type_php3"                    ),
( "phpsa"                   , "file_type_php3"                    ),
( "phpt"                    , "file_type_php3"                    ),
( "phpunit"                 , "file_type_phpunit"                 ),
( "phtml"                   , "file_type_php3"                    ),
( "pipfile"                 , "file_type_pip"                     ),
( "pkb"                     , "file_type_plsql_package_body"      ),
( "pkg"                     , "file_type_package"                 ),
( "pkh"                     , "file_type_plsql_package_header"    ),
( "pks"                     , "file_type_plsql_package_spec"      ),
( "plantuml"                , "file_type_plantuml"                ),
( "plist"                   , "file_type_config"                  ),
( "png"                     , "file_type_image"                   ),
( "po"                      , "file_type_poedit"                  ),
( "policyfile"              , "file_type_chef"                    ),
( "postcssrc"               , "file_type_postcssconfig"           ),
( "pot"                     , "file_type_powerpoint2"             ),
( "potm"                    , "file_type_powerpoint2"             ),
( "potx"                    , "file_type_powerpoint2"             ),
( "ppa"                     , "file_type_powerpoint2"             ),
( "ppam"                    , "file_type_powerpoint2"             ),
( "pps"                     , "file_type_powerpoint2"             ),
( "ppsm"                    , "file_type_powerpoint2"             ),
( "ppsx"                    , "file_type_powerpoint2"             ),
( "ppt"                     , "file_type_powerpoint2"             ),
( "pptm"                    , "file_type_powerpoint2"             ),
( "pptx"                    , "file_type_powerpoint2"             ),
( "prettierignore"          , "file_type_prettier"                ),
( "prettierrc"              , "file_type_prettier"                ),
( "prisma"                  , "file_type_prisma"                  ),
( "pro"                     , "file_type_prolog"                  ),
( "procfile"                , "file_type_procfile"                ),
( "psd"                     , "file_type_photoshop2"              ),
( "psd1"                    , "file_type_powershell_psd2"         ),
( "psm1"                    , "file_type_powershell_psm2"         ),
( "psmdcp"                  , "file_type_nuget"                   ),
( "pst"                     , "file_type_outlook"                 ),
( "pu"                      , "file_type_plantuml"                ),
( "pub"                     , "file_type_publisher"               ),
( "pug-lintrc"              , "file_type_pug"                     ),
( "puml"                    , "file_type_plantuml"                ),
( "puz"                     , "file_type_publisher"               ),
( "py"                      , "file_type_python"                  ),
( "pyc"                     , "file_type_binary"                  ),
( "pyd"                     , "file_type_binary"                  ),
( "pyo"                     , "file_type_binary"                  ),
( "pyup"                    , "file_type_pyup"                    ),
( "q"                       , "file_type_q"                       ),
( "qbs"                     , "file_type_qbs"                     ),
( "qmldir"                  , "file_type_qmldir"                  ),
( "qt"                      , "file_type_video"                   ),
( "qvd"                     , "file_type_qlikview"                ),
( "qvw"                     , "file_type_qlikview"                ),
( "r"                       , "file_type_r"                       ),
( "ra"                      , "file_type_audio"                   ),
( "rake"                    , "file_type_rake"                    ),
( "rakefile"                , "file_type_rake"                    ),
( "rar"                     , "file_type_zip2"                    ),
( "raw"                     , "file_type_audio"                   ),
( "rb"                      , "file_type_ruby"                    ),
( "re"                      , "file_type_reason"                  ),
( "reg"                     , "file_type_registry"                ),
( "rego"                    , "file_type_rego"                    ),
( "rehypeignore"            , "file_type_rehype"                  ),
( "rehyperc"                , "file_type_rehype"                  ),
( "remarkignore"            , "file_type_remark"                  ),
( "remarkrc"                , "file_type_remark"                  ),
( "renovaterc"              , "file_type_renovate"                ),
( "retextignore"            , "file_type_retext"                  ),
( "retextrc"                , "file_type_retext"                  ),
( "rm"                      , "file_type_video"                   ),
( "rmvb"                    , "file_type_video"                   ),
( "rproj"                   , "file_type_rproj"                   ),
( "rs"                      , "file_type_rust"                    ),
( "rspec"                   , "file_type_rspec"                   ),
( "rt"                      , "file_type_reacttemplate"           ),
( "rust-toolchain"          , "file_type_rust_toolchain"          ),
( "rwd"                     , "file_type_matlab"                  ),
( "sailsrc"                 , "file_type_sails"                   ),
( "sass"                    , "file_type_sass"                    ),
( "sbt"                     , "file_type_sbt"                     ),
( "scala"                   , "file_type_scala"                   ),
( "scpt"                    , "file_type_binary"                  ),
( "scptd"                   , "file_type_binary"                  ),
( "scssm"                   , "file_type_scss"                    ),
( "sentryclirc"             , "file_type_sentry"                  ),
( "sequelizerc"             , "file_type_sequelize"               ),
( "sfd"                     , "file_type_font"                    ),
( "sh"                      , "file_type_shell"                   ),
( "sig"                     , "file_type_onenote"                 ),
( "sketch"                  , "file_type_sketch"                  ),
( "slddc"                   , "file_type_matlab"                  ),
( "sldm"                    , "file_type_powerpoint2"             ),
( "sldx"                    , "file_type_powerpoint2"             ),
( "sln"                     , "file_type_sln2"                    ),
( "sls"                     , "file_type_saltstack"               ),
( "slx"                     , "file_type_matlab"                  ),
( "smv"                     , "file_type_matlab"                  ),
( "snyk"                    , "file_type_snyk"                    ),
( "so"                      , "file_type_binary"                  ),
( "solidarity"              , "file_type_solidarity"              ),
( "spe"                     , "file_type_spacengine"              ),
( "sql"                     , "file_type_sql"                     ),
( "sqlite"                  , "file_type_sqlite"                  ),
( "sqlite3"                 , "file_type_sqlite"                  ),
( "src"                     , "file_type_cert"                    ),
( "sss"                     , "file_type_sss"                     ),
( "sst"                     , "file_type_cert"                    ),
( "stl"                     , "file_type_cert"                    ),
( "storyboard"              , "file_type_storyboard"              ),
( "stylelintcache"          , "file_type_stylelint"               ),
( "stylelintignore"         , "file_type_stylelint"               ),
( "stylelintrc"             , "file_type_stylelint"               ),
( "svg"                     , "file_type_svg"                     ),
( "svi"                     , "file_type_video"                   ),
( "svnignore"               , "file_type_subversion"              ),
( "swc"                     , "file_type_flash"                   ),
( "swf"                     , "file_type_flash"                   ),
( "swift"                   , "file_type_swift"                   ),
( "tar"                     , "file_type_zip2"                    ),
( "tcl"                     , "file_type_tcl"                     ),
( "texi"                    , "file_type_tex"                     ),
( "tf"                      , "file_type_terraform"               ),
( "tfignore"                , "file_type_tfs"                     ),
( "tfstate"                 , "file_type_terraform"               ),
( "tgz"                     , "file_type_zip2"                    ),
( "tiff"                    , "file_type_image"                   ),
( "tikz"                    , "file_type_tex"                     ),
( "tlg"                     , "file_type_log"                     ),
( "tmlanguage"              , "file_type_xml"                     ),
( "todo"                    , "file_type_todo"                    ),
( "toml"                    , "file_type_toml"                    ),
( "ts"                      , "file_type_typescript"              ),
( "tst"                     , "file_type_test"                    ),
( "tt2"                     , "file_type_tt"                      ),
( "tta"                     , "file_type_audio"                   ),
( "ttf"                     , "file_type_font"                    ),
( "txt"                     , "file_type_text"                    ),
( "unibeautifyrc"           , "file_type_unibeautify"             ),
( "unity"                   , "file_type_shaderlab"               ),
( "vagrantfile"             , "file_type_vagrant"                 ),
( "vala"                    , "file_type_vala"                    ),
( "vapi"                    , "file_type_vapi"                    ),
( "vash"                    , "file_type_vash"                    ),
( "vbhtml"                  , "file_type_vbhtml"                  ),
( "vbproj"                  , "file_type_vbproj"                  ),
( "vcxproj"                 , "file_type_vcxproj"                 ),
( "vercelignore"            , "file_type_zeit"                    ),
( "vimrc"                   , "file_type_vim"                     ),
( "vob"                     , "file_type_video"                   ),
( "vox"                     , "file_type_audio"                   ),
( "vscodeignore"            , "file_type_vscode-insiders"         ),
( "vsix"                    , "file_type_vsix"                    ),
( "vsixmanifest"            , "file_type_vsixmanifest"            ),
( "vuerc"                   , "file_type_vueconfig"               ),
( "wasm"                    , "file_type_wasm"                    ),
( "watchmanconfig"          , "file_type_watchmanconfig"          ),
( "wav"                     , "file_type_audio"                   ),
( "webm"                    , "file_type_video"                   ),
( "webp"                    , "file_type_webp"                    ),
( "wll"                     , "file_type_word2"                   ),
( "wma"                     , "file_type_audio"                   ),
( "wmv"                     , "file_type_video"                   ),
( "woff"                    , "file_type_font"                    ),
( "woff2"                   , "file_type_font"                    ),
( "wxml"                    , "file_type_wxml"                    ),
( "wxss"                    , "file_type_wxss"                    ),
( "xcodeproj"               , "file_type_xcode"                   ),
( "xfl"                     , "file_type_xfl"                     ),
( "xib"                     , "file_type_xib"                     ),
( "xlf"                     , "file_type_xliff"                   ),
( "xliff"                   , "file_type_xliff"                   ),
( "xls"                     , "file_type_excel2"                  ),
( "xlsm"                    , "file_type_excel2"                  ),
( "xlsx"                    , "file_type_excel2"                  ),
( "xsf"                     , "file_type_infopath"                ),
( "xsn"                     , "file_type_infopath"                ),
( "xtp2"                    , "file_type_infopath"                ),
( "xvc"                     , "file_type_matlab"                  ),
( "xz"                      , "file_type_zip2"                    ),
( "yaml"                    , "file_type_yaml"                    ),
( "yamllint"                , "file_type_yamllint"                ),
( "yarn-integrity"          , "file_type_yarn"                    ),
( "yarnclean"               , "file_type_yarn"                    ),
( "yarnignore"              , "file_type_yarn"                    ),
( "yarnrc"                  , "file_type_yarn"                    ),
( "yaspellerrc"             , "file_type_yandex"                  ),
( "yml"                     , "file_type_yaml"                    ),
( "yy"                      , "file_type_gamemaker2"              ),
( "yyp"                     , "file_type_gamemaker2"              ),
( "zip"                     , "file_type_zip2"                    ),
( "zipx"                    , "file_type_zip2"                    ),
( "zst"                     , "file_type_zip2"                    ),
// SEE ./README on how to edit this file
]
