// SEE ./README on how to edit this file
[
    ( "gradle.kts"                 , "file_type_gradle"                ),
    ( "tar.gz"                     , "file_type_zip"                   ),
    ( "tar.xz"                     , "file_type_zip"                   ),
    ( "tar.zst"                    , "file_type_zip"                   ),
]
